<?php
/**
 * Cobalt by MintJoomla
 * a component for Joomla! 1.7 - 2.5 CMS (http://www.joomla.org)
 * Author Website: http://www.mintjoomla.com/
 * @copyright Copyright (C) 2012 MintJoomla (http://www.mintjoomla.com). All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */

defined('_JEXEC') or die();

jimport('joomla.application.component.modeladmin');

class CobaltModelPack extends JModelAdmin
{
	private $_xml_paths = array();

	public function __construct($config)
	{
		JApplication::registerEvent('onContentBeforeDelete', 'ZipRemover');
		$config['event_before_delete'] = 'deleteZip';
		return parent::__construct($config);
	}

	public function getTable($type = 'Packs', $prefix = 'CobaltTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}


	public function getForm($data = array(), $loadData = true)
	{
		$app = JFactory::getApplication();

		$form = $this->loadForm('com_cobalt.pack', 'pack', array('control' => 'jform', 'load_data' => $loadData));
		if(empty($form))
		{
			return false;
		}
		return $form;
	}

	protected function loadFormData()
	{
		$data = JFactory::getApplication()->getUserState('com_cobalt.pack.edit.'.$this->getName().'.data', array());

		if(empty($data))
		{
			$data = $this->getItem();
		}

		return $data;
	}

	public function save($data)
	{
		return parent::save($data);
	}

	protected function canDelete($record)
	{
		$user = JFactory::getUser();

		return $user->authorise('core.delete', 'com_cobalt.pack.'.(int) $record->id);
	}

	protected function canEditState($record)
	{
		$user = JFactory::getUser();

		return $user->authorise('core.edit.state', 'com_cobalt.pack.'.(int) $record->id);
	}

	public function delete($pks)
	{
		$result = parent::delete($pks);

		if($result)
		{
			$db = JFactory::getDbo();
			$db->setQuery("DELETE FROM #__js_res_packs_sections WHERE pack_id IN (".implode(',', $pks).")");
			$db->query();
		}
	}

	public function build($pack_id)
	{
		jimport('joomla.filesystem.archive');
		JModel::addIncludePath(JPATH_ROOT.DS.'components'.DS.'com_cobalt'.DS.'models');
		$db = & JFactory::getDBO();

		$packsections_model = JModel::getInstance('Packsections', 'CobaltModel');

		$this->pack = $this->getItem($pack_id);
		$pack_sections = $packsections_model->getPackSectoins($pack_id);
		if(!$pack_sections)
		{
			JError::raiseWarning(404, JText::sprintf('CPACKNOSECTIONS', $this->pack->name));
			return false;
		}

		$section_ids = array_keys($pack_sections);

		$type_ids = $type_ids_content = array(0);
		$tpl = $rating = $types_to_save = array();

		define('VIEWS_ROOT', JPATH_ROOT.DS.'components'.DS.'com_cobalt'.DS.'views');
		define('PACK_KEY', $this->pack->key);
		define('PACK_ROOT', JPATH_CACHE.DS.$this->pack->key);

		$name_part = $this->pack->addkey ? '_'.PACK_KEY : '';
		define('NAME_PART', $name_part);

		if(JFolder::exists(PACK_ROOT))
		{
			JFolder::delete(PACK_ROOT);
		}

		JFolder::create(PACK_ROOT);
		$tpl = array();
		foreach ($pack_sections as $k => $ps)
		{
			$section = ItemsStore::getSection($ps->section_id);
			if(is_array($section->params))
			{
				$p = new JRegistry();
				$p->loadArray($section->params);
				$section->params = $p;
			}
			$tpl = array_merge($tpl, $this->_prepare_templates($ps->params, $section->params, 'general.'));

			$cats = $this->_getItems('categories', array('section_id = '.$ps->section_id));
			foreach ($cats as $c)
			{
				$cat_params = new JRegistry($c->params);
				$tpl = array_merge($tpl, $this->_prepare_templates($ps->params, $cat_params));
				$c->params = $cat_params->toString();
				$categories[$c->id] = $c;

				if($c->image)
				{
					$icons[] = $c->image;
				}
			}

			$types = $ps->params->get('types', array());

			foreach ($types as $type_id => $type_settings)
			{
				$type_ids[] = $type_id;
				$type = ItemsStore::getType($type_id);
				if(!is_object($type->params))
				{
					$type->params = new JRegistry($type->params);
				}
				$type_settings = new JRegistry($type_settings);
				if($type_settings->get('copy_content'))
				{
					$type_ids_content[] = $type_id;
				}
				if($type_settings->get('article'))
				{
					$tpl[] = 'record'.DS.'tmpl'.DS.'default_record_'.$type->params->get('properties.tmpl_article');
					$type->params->set('properties.tmpl_article', $this->_getTmplName( $type->params->get('properties.tmpl_article')));
				}
				if($type_settings->get('articleform'))
				{
					$tpl[] = 'form'.DS.'tmpl'.DS.'default_form_'.$type->params->get('properties.tmpl_articleform');
					$type->params->set('properties.tmpl_articleform', $this->_getTmplName($type->params->get('properties.tmpl_articleform')));
				}
				if($type_settings->get('rating'))
				{
					$rating[] = $type->params->get('properties.tmpl_rating');
				}
				if($type_settings->get('comment'))
				{
					$tpl[] = 'record'.DS.'tmpl'.DS.'default_comments_'.$type->params->get('properties.tmpl_comment');
					$type->params->set('properties.tmpl_comment', $this->_getTmplName($type->params->get('properties.tmpl_comment')));
				}
				if($type_settings->get('categoryselect'))
				{
					$tpl[] = 'form'.DS.'tmpl'.DS.'default_category_'.$type->params->get('properties.tmpl_categoryselect');
					$type->params->set('properties.tmpl_categoryselect', $this->_getTmplName($type->params->get('properties.tmpl_categoryselect')));
				}
				$type->params = $type->params->toString();
				$types_to_save[$type->id] = $type;
			}
			$section->params = $section->params->toString();
			$sections[$section->id] = $section;
		}

		$tpl = array_unique($tpl);
		foreach ($tpl AS $tplname)
		{
			$this->_copy_tmpl($tplname);
		}
		$rating = array_unique($rating);
		foreach ($rating AS $tplname)
		{
			$this->_copy_rating_tmpl($tplname);
		}

		$this->_packFile('categories',$categories);
		$this->_packFile('sections', $sections);

		$user_categories = $this->_getItems('category_user', array('section_id IN ( '.implode(',', $section_ids).')'));
		settype($user_categories, 'array');
		foreach ($user_categories as $uc)
		{
			if($uc->icon)
			{
				$dest = PACK_ROOT.DS.'admin'.DS.PACK_KEY.DS.'usercategories'.DS.$uc->user_id.DS.$uc->icon;
				$folder = dirname($dest);
				if(!JFolder::exists($folder))
				{
					JFolder::create($folder);
				}
				JFile::copy(JPATH_ROOT.DS.'images'.DS.'usercategories'.DS.$uc->user_id.DS.$uc->icon, $dest);

// 				$icons[] = 'images'.DS.'usercategories'.DS.$uc->user_id.DS.$uc->icon;
			}
		}
		$this->_packFile('category_user', $user_categories);

		$this->_packFile('types', $types_to_save);
		$fields = $this->_getItems('fields', array('type_id IN ('.implode(',', $type_ids).')'));
		$this->_packFile('fields', $fields);
		$this->_packFile('fields_group', $this->_getItems('fields_group', array('type_id IN ('.implode(',', $type_ids).')')));

		$records = $this->_getItems('record', array('type_id IN ('.implode(',', $type_ids_content).') AND section_id IN ( '.implode(',', $section_ids).')'));
		$this->_packFile('records', $records);

		$records_comments_ids = array_keys($records);

		$records[0] = 0;
		$record_ids = array_keys($records);

		$users = $this->_getUsers($record_ids);
		$this->_packFile('users', $users);
		$users[0] = 1;
		$user_ids = array_keys($users);

		$comments = array();
		if(!empty($records_comments_ids))
		{
			$comments = $this->_getItems('comments', array('record_id IN ('.implode(',', $records_comments_ids).')'));
		}
		$this->_packFile('comments', $comments);

		$comments[0] = 0;
		$comment_ids = array_keys($comments);


		$votes = $this->_getItems('vote', array('(ref_id IN ('.implode(',', $record_ids).') AND ref_type = "record") OR (ref_id IN ('.implode(',', $comment_ids).') AND ref_type = "comment")'));
		$this->_packFile('vote', $votes);

		$tags_history = $this->_getItems('tags_history', array('record_id IN ('.implode(',', $record_ids).')'));
		$this->_packFile('tags_history', $tags_history);

		$tags = $this->_getItems('tags', array('id IN ( SELECT tag_id FROM #__js_res_tags_history WHERE record_id IN ('.implode(',', $record_ids).'))'));
		$this->_packFile('tags', $tags);

		$this->_packFile('favorite', $this->_getItems('favorite', array('record_id IN ('.implode(',', $record_ids).')')));
		$this->_packFile('record_values', $this->_getItems('record_values', array('record_id IN ('.implode(',', $record_ids).')')));
		$this->_packFile('record_category', $this->_getItems('record_category', array('record_id IN ('.implode(',', $record_ids).')')));
		$this->_packFile('sales', $this->_getItems('sales', array('record_id IN ('.implode(',', $record_ids).')')));

		$this->_packFile('notifications', $this->_getItems('notifications', array('ref_2 IN ( '.implode(',', $section_ids).') AND user_id IN('.implode(',', $user_ids).')')));
		$this->_packFile('subscriptions', $this->_getItems('subscriptions', array('section_id IN ( '.implode(',', $section_ids).') AND user_id IN('.implode(',', $user_ids).')')));
		$this->_packFile('subscription_params', $this->_getItems('subscription_params', array('section_id IN ( '.implode(',', $section_ids).') AND user_id IN('.implode(',', $user_ids).')')));
		$this->_packFile('moderators', $this->_getItems('moderators', array('section_id IN ( '.implode(',', $section_ids).') AND user_id IN('.implode(',', $user_ids).')')));

		$files = $this->_getItems('files', array('record_id IN ('.implode(',', $record_ids).')'));
		$this->_packFile('files',$files);
		settype($files, 'array');
		$cobalt_params = JComponentHelper::getParams('com_cobalt');
		foreach ($files as $file)
		{
			$field->params = new JRegistry($fields[$file->field_id]->params);
			$subfolder = $field->params->get('params.subfolder', JFile::getExt($file->filename));
			$dest = PACK_ROOT.DS.'admin'.DS.PACK_KEY.DS.'files'.DS.$subfolder.DS.$file->fullpath;
			$folder = dirname($dest);
			if(!JFolder::exists($folder))
			{
				JFolder::create($folder);
			}
			JFile::copy(JPATH_ROOT.DS.$cobalt_params->get('general_upload').DS.$subfolder.DS.$file->fullpath, $dest);
		}
		$add_files = explode("\n", $this->pack->add_files);
		settype($add_files, 'array');
		if($icons) $add_files = array_merge($add_files, $icons);
		if(!empty($add_files))
		{
			foreach ($add_files AS $file)
			{
				$file = trim($file);
				if(!$file) continue;
				$path = JPATH_ROOT.DS.$file;
				$path = JPath::clean($path);

				if(JFile::exists($path))
				{
					$folder = JPath::clean(PACK_ROOT.DS.'add'.DS.$file);
					JFolder::create(dirname($folder));
					JFile::copy($path, $folder);
				}
				if(JFolder::exists($path))
				{
					JFolder::copy($path, JPath::clean(PACK_ROOT.DS.'add'.DS.$file), null, TRUE);
				}
			}
			$this->_add_folder_path_additionals();
		}

		foreach ($pack_sections as $ps)
		{
			$ps->params = $ps->params->toString();
		}


		$this->pack->sections = $pack_sections;
		$this->_packFile('pack', $this->pack);

		JFile::copy(JPATH_COMPONENT_ADMINISTRATOR.DS.'library'.DS.'php'.DS.'pack'.DS.'install.php', PACK_ROOT.DS.'install.pack.php');

		$this->_generateXml();

		//Archiving
		$zip_filename = JPATH_CACHE.DS.'pack_'.PACK_KEY.'.build.7.'.($this->pack->version + 1).'.zip';

		$zipper = new Zipper();
		$zipper->open($zip_filename, ZIPARCHIVE::OVERWRITE);
		$zipper->addDir(PACK_ROOT);
		$zipper->close();

		JFolder::delete(PACK_ROOT);

		$table = $this->getTable();
		$table->load($this->pack->id);
		$table->btime = JFactory::getDate()->toMySQL();
		$table->version += 1;
		$table->store();

		return true;
	}

	private function _getTmplName($name)
	{
		if($this->pack->addkey)
		{
			return str_replace(NAME_PART, '', $name).NAME_PART;
		}
		else
		{
			return $name;
		}
	}

	private function _copy_tmpl($name)
	{
		$name = str_replace(NAME_PART, '', $name);
		$src = JPATH_ROOT.DS.'components'.DS.'com_cobalt'.DS.'views'.DS.$name;
		$dest = PACK_ROOT.DS.'site'.DS.'views'.DS.$name.NAME_PART;

		$folder = dirname($dest);

		if(!JFolder::exists($folder))
		{
			JFolder::create($folder);
		}

		JFile::copy($src.'.php', $dest.'.php');
		$this->_add_file_path('views'.DS.$name.NAME_PART.'.php');

		if(JFile::exists($src.'.css'))
		{
			JFile::copy($src.'.css', $dest.'.css');
			$this->_add_file_path('views'.DS.$name.NAME_PART.'.css');
		}
		if(JFile::exists($src.'.json'))
		{
			JFile::copy($src.'.json', $dest.'.json');
			$this->_add_file_path('views'.DS.$name.NAME_PART.'.json');
		}
		if(JFile::exists($src.'.png'))
		{
			JFile::copy($src.'.png', $dest.'.png');
			$this->_add_file_path('views'.DS.$name.NAME_PART.'.png');
		}
		if(JFile::exists($src.'.xml'))
		{
			JFile::copy($src.'.xml', $dest.'.xml');
			$this->_add_file_path('views'.DS.$name.NAME_PART.'.xml');
		}
		if(JFile::exists($src.'.js'))
		{
			JFile::copy($src.'.js', $dest.'.js');
			$this->_add_file_path('views'.DS.$name.NAME_PART.'.js');
		}
		if(JFolder::exists($src))
		{
			JFolder::copy($src, $dest, '', TRUE);
			$this->_add_folder_path('views'.DS.$name.NAME_PART);
		}
	}

	private function _copy_rating_tmpl($name)
	{
		$src = JPATH_ROOT.DS.'components'.DS.'com_cobalt'.DS.'views'.DS.'rating_tmpls'.DS;
		$dest = PACK_ROOT.DS.'site'.DS.'views'.DS.'rating_tmpls'.DS;

		$folder = dirname($dest);

		if(!JFolder::exists($folder))
		{
			JFolder::create($folder);
		}

		if(JFolder::exists($src.$name.'_img'))
		{
			JFolder::copy($src.$name.'_img', $dest.$name.'_img', '', TRUE);
			$this->_add_folder_path('views'.DS.'rating_tmpls'.DS.$name.'_img');
		}

		JFile::copy($src.'rating_'.$name.'.php', $dest.'rating_'.$name.'.php');
		$this->_add_file_path('views'.DS.'rating_tmpls'.DS.'rating_'.$name.'.php');

		if(JFile::exists($src.'rating_'.$name.'.json'))
		{
			JFile::copy($src.'rating_'.$name.'.json', $dest.'rating_'.$name.'.json');
			$this->_add_file_path('views'.DS.'rating_tmpls'.DS.'rating_'.$name.'.json');
		}

		JFile::copy($src.'rating_'.$name.'.xml', $dest.'rating_'.$name.'.xml');
		$this->_add_file_path('views'.DS.'rating_tmpls'.DS.'rating_'.$name.'.xml');
	}

	private function _prepare_templates($params, &$tmpls, $prefix = NULL)
	{
		$tpl = array();
		if($params->get('list') && $tmpls->get($prefix.'tmpl_list', array()))
		{
			$list = $tmpls->get($prefix.'tmpl_list', array());
			settype($list, 'array');
			foreach ($list AS $tplname)
			{
				if(!$tplname)
				{
					continue;
				}
				$tpl[] = 'records'.DS.'tmpl'.DS.'default_list_'.$tplname;
				$tpl_names[] =  $this->_getTmplName($tplname);
			}
			if($tpl)
			{
				$tmpls->set($prefix.'tmpl_list', $tpl_names);
			}
		}
		if($params->get('filters') && $tmpls->get($prefix.'tmpl_filters'))
		{
			$tpl[] = 'records'.DS.'tmpl'.DS.'default_filters_'.$tmpls->get($prefix.'tmpl_filters');
			$tmpls->set($prefix.'tmpl_filters', $this->_getTmplName($tmpls->get($prefix.'tmpl_filters')));
		}
		if($params->get('cat_index') && $tmpls->get($prefix.'tmpl_category'))
		{
			$tpl[] = 'records'.DS.'tmpl'.DS.'default_cindex_'.$tmpls->get($prefix.'tmpl_category');
			$tmpls->set($prefix.'tmpl_category', $this->_getTmplName($tmpls->get($prefix.'tmpl_category')));
		}
		if($params->get('menu') && $tmpls->get($prefix.'tmpl_menu'))
		{
			$tpl[] = 'records'.DS.'tmpl'.DS.'default_menu_'.$tmpls->get($prefix.'tmpl_menu');
			$tmpls->set($prefix.'tmpl_menu', $this->_getTmplName($tmpls->get($prefix.'tmpl_menu')));
		}
		if($params->get('compare') && $tmpls->get($prefix.'tmpl_compare'))
		{
			$tpl[] = 'records'.DS.'tmpl'.DS.'default_list_'.$tmpls->get($prefix.'tmpl_compare');
			$tmpls->set($prefix.'tmpl_compare', $this->_getTmplName($tmpls->get($prefix.'tmpl_compare')));
		}
		if($params->get('markup') && $tmpls->get($prefix.'tmpl_markup'))
		{
			$tpl[] = 'records'.DS.'tmpl'.DS.'default_markup_'.$tmpls->get($prefix.'tmpl_markup');
			$tmpls->set($prefix.'tmpl_markup', $this->_getTmplName($tmpls->get($prefix.'tmpl_markup')));
		}
		return $tpl;
	}

	private function _packFile($filename, $object)
	{
		if(!$object) $object = array();

		JFile::write(PACK_ROOT.DS.'admin'.DS.PACK_KEY.DS.$filename.'.json', json_encode($object));
	}

	private function _getItems($table_name, $where = array())
	{
		$db = JFactory::getDbo();
		$db->setQuery("SELECT * FROM #__js_res_{$table_name} WHERE ". implode(' AND ', $where). " ORDER BY id");
		$items = $db->loadObjectList('id');
		return $items;
	}

	private function _getUsers($record_ids)
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
		$query->select('*');
		$query->from('#__users');
		$query->where('id IN (SELECT user_id FROM #__js_res_record WHERE id IN ('.implode(',', $record_ids).') )
		OR id IN(SELECT user_id FROM #__js_res_comments WHERE record_id IN ('.implode(',', $record_ids).') )');
		$db->setQuery($query);
		$users = $db->loadObjectList('id');

		return $users;
	}

	private function _add_file_path($name, $index = 'file')
	{
		$this->_xml_paths[$index][] = $name;
	}

	private function _add_folder_path($path)
	{
		$dest = PACK_ROOT.DS.'site'.DS;

		$files = JFolder::files($dest.$path);

		foreach ($files AS $file)
		{
			$this->_xml_paths['file'][] = $path.DS.$file;
		}

		if($folders = JFolder::folders($dest.DS.$path))
		{
			foreach ($folders as $folder) {
				$this->_add_folder_path($path.DS.$folder);
			}
		}

		$this->_xml_paths['folder'][] = $path;
	}

	private function _add_folder_path_additionals()
	{
		$dest = PACK_ROOT.DS.'add';
		if(!JFolder::exists($dest))
		{
			return;
		}

		$files = JFolder::files($dest);

		foreach ($files AS $file)
		{
			$this->_xml_paths['add'][] = $file;
		}

		if($folders = JFolder::folders($dest))
		{
			foreach ($folders as $folder) {
				$this->_xml_paths['addfolder'][] = $folder;
			}
		}
	}

	private function _generateXml()
	{
		$install = JFile::read(JPATH_COMPONENT_ADMINISTRATOR.DS.'library'.DS.'php'.DS.'pack'.DS.'install.xml.txt');

		$install = str_replace('[NAME]', $this->pack->get('name', 'Pack name'), $install);
		$install = str_replace('[AUTHOR_NAME]', $this->pack->get('author_name', 'Author name not set'), $install);
		$install = str_replace('[AUTHOR_EMAIL]', $this->pack->get('author_email', 'Author email  not set'), $install);
		$install = str_replace('[AUTHOR_URL]', $this->pack->get('author_url', 'Author url  not set'), $install);
		$install = str_replace('[CTIME]', $this->pack->get('ctime'), $install);
		$install = str_replace('[COPYRIGHT]', $this->pack->get('copyright', ''), $install);
		$install = str_replace('[VERSION]', (int)$this->pack->get('version', '') + 1, $install);
		$install = str_replace('[DESCR]', $this->pack->get('description', ''), $install);
		$install = str_replace('[KEY]', $this->pack->get('key'), $install);

		$replace = '';
		if(!empty($this->_xml_paths['file']))
		{
			foreach ($this->_xml_paths['file'] AS $file)
			{
				$replace .= "\t\t\t<filename>{$file}</filename>\r\n";
			}
		}

		if(!empty($this->_xml_paths['folder']))
		{
			foreach ($this->_xml_paths['folder'] AS $folder)
			{
				$replace .= "\t\t\t<folder>{$folder}</folder>\r\n";
			}
		}

		$install = str_replace('[FRONT]', $replace, $install);
		$replace = '';
		if(isset($this->_xml_paths['add']))
		{
			foreach ($this->_xml_paths['add'] AS $file)
			{
				$replace .= "\t\t\t<filename>{$file}</filename>\r\n";
			}
		}
		if(isset($this->_xml_paths['addfolder']))
		{
			foreach ($this->_xml_paths['addfolder'] AS $folder)
			{
				$replace .= "\t\t\t<folder>{$folder}</folder>\r\n";
			}
		}
		if($replace)
		{
			$replace = '<files folder="add">'.$replace.'</files>';
		}
		$install = str_replace('[ADD]', $replace, $install);

		JFile::write(PACK_ROOT.DS.'pack.xml', $install);
	}
}

class ZipRemover extends JEvent
{
	public function deleteZip($context, $table)
	{

		$filename = JPATH_ADMINISTRATOR.DS.'cache'.DS.'pack_'.$table->key.'.zip';
		if(JFile::exists($filename))
		{
			JFile::delete($filename);
		}
	}
}

class Zipper extends ZipArchive
{
	public function addDir($path) {
		$nodes = glob($path . '/*');
		foreach ($nodes as $node) {
			if (is_dir($node)) {
				$this->addDir($node);
			} else if (is_file($node))  {
				$this->addFile($node, str_replace(PACK_ROOT, PACK_KEY, $node));
			}
		}
	}

}