<?php
/**
 * Cobalt by MintJoomla
 * a component for Joomla! 1.7 - 2.5 CMS (http://www.joomla.org)
 * Author Website: http://www.mintjoomla.com/
 * @copyright Copyright (C) 2012 MintJoomla (http://www.mintjoomla.com). All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */
defined('_JEXEC') or die();
require_once JPATH_ROOT . DS . 'components/com_cobalt/library/php/fields/cobaltfield.php';

require_once JPATH_ROOT . DS . 'components' . DS . 'com_cobalt' . DS . 'library' . DS . 'php' . DS . 'cdate.php';

class JFormFieldCDatetime extends CFormField
{
	public function __construct($field, $default)
	{
		parent::__construct($field, $default);
		switch($this->params->get('params.template_input'))
		{
			case 'multiple_dates_picker.php': $type = 'multiple'; break;
			case 'range_date_picker.php': $type = 'range'; break;
			case 'simple_select.php': $type = 'select'; break;
			case 'single_date_picker.php': $type = 'single'; break;
			case 'text_input.php': $type = 'input'; break;
			case 'text_input_picker.php': $type = 'input_picker'; break;
		}
		switch($this->params->get('params.template_filter'))
		{
			case 'range_date_picker.php': $filter = 'range'; break;
			case 'single_date_picker.php': $filter = 'single'; break;
			case 'compare.php': $filter = 'compare'; break;
		}
		
		$this->params->set('params.type', $type);
		$this->params->set('params.filter_style', $filter);
	}
	
	public function getInput()
	{
		$doc = JFactory::getDocument();
		
		$path = JURI::root() . 'media/mint/js/moodatepicker/';
		$lang = JFactory::getLanguage()->getTag();
		if (JFile::exists($path . 'Locale.' . $lang . '.DatePicker.js'))
			$doc->addScript($path . 'Locale.' . $lang . '.DatePicker.js');
		else
			$doc->addScript($path . 'Locale.en-GB.DatePicker.js');
		$doc->addScript($path . 'Picker.js');
		$doc->addScript($path . 'Picker.Attach.js');
		$doc->addScript($path . 'Picker.Date.js');
		$doc->addStyleSheet($path . 'datepicker.css');
		$doc->addStyleSheet(JURI::root() . 'components/com_cobalt/fields/datetime/datetime.css');
		$doc->addScript(JURI::root() . 'components/com_cobalt/fields/datetime/datetime.js');
		if($this->params->get('params.template_input') == 'range_date_picker.php')
				$doc->addScript($path . 'Picker.Date.Range.js');
		
				
		$format = $this->params->get('params.format', '%d %b %Y');
		if ($format == 'custom' && $this->params->get('params.custom', '') != '')
		{
			$format = $this->params->get('params.custom', '');
		}
		
		settype($this->value, 'array');
		
		$this->format = $format;
		
		$default = '';
		if(in_array($this->params->get('params.template_input'), array('simple_select.php', 'text_input_picker.php', 'text_input.php')))
		{
			if ($this->params->get('params.input_default', '1 day'))
			{
				switch ($this->params->get('params.input_default', '1 day'))
				{
					case 'now' :
						$sql = 'SELECT NOW()';
						break;
					case 'custom' :
						if ($this->params->get('params.custom_input'))
							$sql = "SELECT DATE_ADD(NOW(), INTERVAL {$this->params->get('params.custom_input')})";
						else
							$sql = 'SELECT NOW()';
						break;
					default :
						$sql = "SELECT DATE_ADD(NOW(), INTERVAL {$this->params->get('params.input_default', '1 day')})";
				}
				$db = JFactory::getDbo();
				$db->setQuery($sql);
				$default = $db->loadResult();
			}
		}
		
		$this->default = $default;
		
		return $this->_display_input();		
		
	}

	public function validate($value, $record, $type, $section)
	{
		if ($value[0] && in_array($this->params->get('params.type', 'single'), array('input', 'input_picker') ))
		{
			if ($this->params->get('params.input_order', 'month') == 'month')
				$format = sprintf('m%sd%sY',$this->params->get('params.input_delimiter', '/'), $this->params->get('params.input_delimiter', '/') );
			else
				$format = sprintf('d%sm%sY',$this->params->get('params.input_delimiter', '/'), $this->params->get('params.input_delimiter', '/') );
			$format .= $this->params->get('params.time') ? 'H:i:s' : '00:00:00';
			$cdate = new CDate();
			if ($date = $cdate->strtotime($value[0], $format))
			{
			}
			elseif ($date = strtotime($value[0]))
			{
			}			
			if (!$date)
			{
				$this->setError('Wrong date input');
			}
		}
		if ($this->params->get('params.type', 'single') == 'select')
		{
			$f = $this->params->get('params.time') ? 'Y-m-d H:i:s' : 'Y-m-d 00:00:00';
			$val = $value[0];
			$val = sprintf('%s-%s-%s %s:%s', str_pad($val['year'], 4, 0, STR_PAD_LEFT), str_pad($val['month'], 2, 0, STR_PAD_LEFT), str_pad($val['day'], 2, 0, STR_PAD_LEFT), isset($val['hour']) ? str_pad($val['hour'], 2, 0, STR_PAD_LEFT) : '00', isset($val['min']) ? str_pad($val['min'], 2, 0, STR_PAD_LEFT) : '00');
			
			if ($val = strtotime($val))
			{
				$value = array(date($f, $val));
			}
			else
				$value = '';
		}
		parent::validate($value, $record, $type, $section);
	}

	public function onJSValidate()
	{
		$js = '';
		
		$js .= "\n\t\t" . 'var field = $$("[name^=\"jform[fields][' . $this->id . ']\"]");
		var value = 0;
		Array.each(field, function(item, index){
			if(item.value != "")
				value ++;
		});';
		if ($this->required)
		{
			$js .= "\n\t\tif(!value){hfid.push({$this->id}); isValid = false; errorText.push('" . JText::sprintf('CFIELDREQUIRED', $this->label) . "');}";
		}
		return $js;
	}

	public function onFilterWornLabel($section)
	{
		$out = '';
		if(is_array($this->value) && array_key_exists('range', $this->value) && !empty($this->value['range']))
		{
			$value = explode(',', trim($this->value['range']));
			ArrayHelper::clean_r($value);
			$start = explode(' ', $value[0]);
			if(empty($start[1]))
			{
				$start[1] = '00:00:00';
			}

			if(empty($value[1]))
			{
				$value[1] = $value[0];
			}
			$end = explode(' ',  $value[1]);
			if(empty($end[1]))
			{
				$end[1] = '23:59:59';
			}

			$end = $this->_getFormatted(implode(' ', $end), true);
			$start = $this->_getFormatted(implode(' ', $start), true);

			$out .= $start .' - '. $end;

		}
		elseif(is_array($this->value) && array_key_exists('condition', $this->value) && !empty($this->value['condition']) && !empty($this->value['date']))
		{

			$value = $this->_getFormatted($this->value['date'], true);

			$opt[1] = JText::_($this->params->get('params.filter_compare_stbefore'));
			$opt[2] = JText::_($this->params->get('params.filter_compare_stafter'));
			$opt[3] = JText::_($this->params->get('params.filter_compare_endbefore'));
			$opt[4] = JText::_($this->params->get('params.filter_compare_endafter'));

			$out .= $opt[$this->value['condition']] . ' ' . $value;
		}
		elseif(!empty($this->value) && is_string($this->value))
		{
			$out .= $this->_getFormatted($this->value, true);
		}

		return $out;
	}

	public function onFilterWhere($section, &$query)
	{
		$db = JFactory::getDbo();
		$sql = $db->getQuery(true);

		$sql->select('v.record_id');
		$sql->from('#__js_res_record_values AS v');
		$sql->where('v.section_id = '.$section->id);
		$sql->where('v.field_key = '.$db->quote($this->key));

		if(is_array($this->value) && array_key_exists('range', $this->value) && !empty($this->value['range']))
		{
			$value = explode(',', trim($this->value['range']));
			ArrayHelper::clean_r($value);
			$start = explode(' ', $value[0]);
			if(empty($start[1]))
			{
				$start[1] = '00:00:00';
			}

			if(empty($value[1]))
			{
				$value[1] = $value[0];
			}
			$end = explode(' ',  $value[1]);
			if(empty($end[1]))
			{
				$end[1] = '23:59:59';
			}

			$end = $db->quote(implode(' ', $end));
			$start = $db->quote(implode(' ', $start));

			switch ($this->params->get('params.type', 'single'))
			{
				case 'multiple':
					$sql->where($start." <= (SELECT MIN(field_value) FROM #__js_res_record_values
							WHERE record_id = v.record_id AND section_id = {$section->id}
							AND field_key = '{$this->key}')");

					$sql->where($end." >= (SELECT MAX(field_value) FROM #__js_res_record_values
							WHERE record_id = v.record_id AND section_id = {$section->id}
							AND field_key = '{$this->key}')");
				case 'range':
					$sql->where($start." <= (SELECT field_value FROM #__js_res_record_values
							WHERE record_id = v.record_id AND section_id = {$section->id}
							AND field_key = '{$this->key}' AND value_index = 0)");

					$sql->where($end." >= (SELECT field_value FROM #__js_res_record_values
							WHERE record_id = v.record_id AND section_id = {$section->id}
							AND field_key = '{$this->key}' AND value_index = 1)");
					break;
				case 'single':
				default :
					$sql->where("v.field_value BETWEEN {$start} - INTERVAL 1 SECOND AND {$end}");
			}
		}
		elseif(is_array($this->value) && array_key_exists('condition', $this->value) && !empty($this->value['condition']) && !empty($this->value['date']))
		{
			$value = $this->value['date'];
			$value = $db->quote($value);

			switch ($this->params->get('params.type', 'single'))
			{
				case 'multiple' :
					switch ($this->value['condition'])
					{
						case 2 :
							$having = "(SELECT MIN(field_value) FROM #__js_res_record_values
									WHERE record_id = v.record_id AND section_id = {$section->id}
									AND field_key = '{$this->key}') >= {$value}";
							break;
						case 3 :
							$having = "(SELECT MAX(field_value) FROM #__js_res_record_values
									WHERE record_id = v.record_id AND section_id = {$section->id}
									AND field_key = '{$this->key}') <= {$value}";
							break;
						case 4 :
							$having = "(SELECT MAX(field_value) FROM #__js_res_record_values
									WHERE record_id = v.record_id AND section_id = {$section->id}
									AND field_key = '{$this->key}') >= {$value}";
							break;
						case 1 :
						default :
							$having = "(SELECT MIN(field_value) FROM #__js_res_record_values
									WHERE record_id = v.record_id AND section_id = {$section->id}
									AND field_key = '{$this->key}') <= {$value}";
							break;
					}
					break;
				case 'range' :
					switch ($this->value['condition'])
					{
						case 2 :
							$sql->where("(v.field_value >=  {$value} AND v.value_index = '0')");
							break;
						case 3 :
							$sql->where("(v.field_value <= {$value} AND v.value_index = '1')");
							break;
						case 4 :
							$sql->where("(v.field_value >= {$value} AND v.value_index = '1')");
							break;
						case 1:
						default :
							$sql->where("(v.field_value <= {$value} AND v.value_index = '0')");
							break;

					}
					break;
				case 'single' :
				default :
					switch ($this->value['condition'])
					{
						case 1 :
						case 3 :
							$sql->where("v.field_value <= {$value}");
							break;
						case 2 :
						case 4 :
							$sql->where("v.field_value >= {$value}");
							break;
						default :
							$sql->where("v.field_value = {$value}");
					}
					break;
			}
		}
		elseif(!empty($this->value) && is_string($this->value))
		{
			$value = $db->quote($this->value);
			switch ($this->params->get('params.type', 'single'))
			{
				case 'range':
					$sql->where("DATE({$value}) BETWEEN
							(SELECT field_value FROM #__js_res_record_values
							WHERE record_id = v.record_id AND section_id = {$section->id}
							AND field_key = '{$this->key}' AND value_index = 0)
						AND (SELECT field_value FROM #__js_res_record_values
							WHERE record_id = v.record_id AND section_id = {$section->id}
							AND field_key = '{$this->key}' AND value_index = 1)");
					break;
				case 'multiple' :
				case 'single' :
				default :
					$sql->where("DATE(v.field_value) = DATE({$value})");
					break;
			}
		}
		else
		{
			return false;
		}

		$ids = $this->getIds((string)$sql);
		$query->where("r.id IN(".implode(',', $ids).")");
		return true;
	}

	public function onRenderFilter($section, $module = false)
	{
		$doc = JFactory::getDocument();
		$path = JURI::root() . 'media/mint/js/moodatepicker/';
		$lang = JFactory::getLanguage()->getTag();
		if (JFile::exists($path . 'Locale.' . $lang . '.DatePicker.js'))
			$doc->addScript($path . 'Locale.' . $lang . '.DatePicker.js');
		else
			$doc->addScript($path . 'Locale.en-GB.DatePicker.js');
		$doc->addScript($path . 'Picker.js');
		$doc->addScript($path . 'Picker.Attach.js');
		$doc->addScript($path . 'Picker.Date.js');
		$doc->addScript($path . 'Picker.Date.Range.js');
		$doc->addStyleSheet($path . 'datepicker.css');
		$doc->addStyleSheet(JURI::root() . 'components/com_cobalt/fields/datetime/datetime.css');
		$doc->addScript(JURI::root() . 'components/com_cobalt/fields/datetime/datetime.js');

		$format = $this->params->get('params.format', '%d %b %Y');
		if ($format == 'custom' && $this->params->get('params.custom', '') != '')
			$format = $this->params->get('params.custom', '');
		$this->format = $format;
		$date = new CDate();
		$this->php_format = $date->convertFormat($format);

		return $this->_display_filter($section, $module);
	}

	public function onPrepareFullTextSearch($value, $record, $type, $section)
	{
		return implode(',', $value);
	}

	public function onPrepareSave($value, $record, $type, $section)
	{
		if (!isset($value[0]) || $value[0] == '')
		{
			return array();
		}		
	
		$format = ($this->params->get('params.format') == 'custom' ? $this->params->get('params.custom') : $this->params->get('params.format'));
		
		$f = $this->params->get('params.time') ? 'Y-m-d H:i:s' : 'Y-m-d 00:00:00';
		if (!in_array($this->params->get('params.type'), array('select', 'multiple', 'range', 'single')))
		{
			if ($this->params->get('params.input_order', 'month') == 'month')
				//$format = 'm-d-Y H:i:s';
				$format = sprintf('m%sd%sY',$this->params->get('params.input_delimiter', '/'), $this->params->get('params.input_delimiter', '/') );
			else
				$format = sprintf('d%sm%sY',$this->params->get('params.input_delimiter', '/'), $this->params->get('params.input_delimiter', '/') );
				//$format = 'd-m-Y H:i:s';
			$format .= $this->params->get('params.time') ? 'H:i:s' : '00:00:00';
			
			$cdate = new CDate();
			
			if ($date = $cdate->strtotime($value[0], $format))
			{
				$value = array(date($f, $date));
			}
			elseif ($date = strtotime($value[0]))
			{
				$value = array(date($f, $date));
			}
		}
		if ($this->params->get('params.type') == 'select')
		{
			$value = $value[0];
			$val = sprintf('%s-%s-%s %s:%s', str_pad($value['year'], 4, 0, STR_PAD_LEFT), str_pad($value['month'], 2, 0, STR_PAD_LEFT), str_pad($value['day'], 2, 0, STR_PAD_LEFT), isset($value['hour']) ? str_pad($value['hour'], 2, 0, STR_PAD_LEFT) : '00', isset($value['min']) ? str_pad($value['min'], 2, 0, STR_PAD_LEFT) : '00');
			
			if ($val = strtotime($val))
			{
				$value = array(date($f, $val));
			}
			else
				$value = array();
		}
		return $value;
	}

	public function onStoreValues($record)
	{
		ArrayHelper::clean_r($this->value);
		if (count($this->value) == 0)
		{
			return;
		}
		
		$db = JFactory::getDbo();
		if ($this->params->get('params.ovr_ctime', 0))
		{
			$patern = ($this->params->get('params.ctime_add') ? "DATE_ADD('%s', INTERVAL " . $this->params->get('params.ctime_add') . ")" : "'%s'");
			if ($this->params->get('params.type', 'single') == 'multiple')
			{
				$value = $this->value;
				sort($value);
			}
			$cdate = sprintf($patern, $this->value[0]);
			$query = "UPDATE #__js_res_record SET ctime = {$cdate} WHERE id = '{$record['id'] }'";
			$db->setQuery($query);
			$db->query();
		}
		if ($this->params->get('params.ovr_extime', 0))
		{
			$patern = ($this->params->get('params.extime_add') ? "DATE_ADD('%s', INTERVAL " . $this->params->get('params.extime_add') . ")" : "'%s'");
			if ($this->params->get('params.type', 'single') == 'multiple')
			{
				$value = $this->value;
				rsort($value);
			}
			
			$exdate = sprintf($patern, $this->value[0]);
			$query = "UPDATE #__js_res_record SET extime = {$exdate} WHERE id = '{$record['id'] }'";
			$db->setQuery($query);
			$db->query();
		}
		return $this->value;
	}

	public function onRenderFull($record, $type, $section)
	{		
		return $this->_render('full', $record, $type, $section);
	}
	
	public function onRenderList($record, $type, $section)
	{
		return $this->_render('list', $record, $type, $section);
	}
	
	private function _render($client, $record, $type, $section )
	{
		if (!$this->value)
			return;
		settype($this->value, 'array');
		
		$dates = array();
		foreach($this->value as $value)
		{		
			$date = $this->_getFormatted($value, false, '_out');
			if ($this->params->get('params.filter_enable'))
			{
				$tip = ($this->params->get('params.filter_tip') ? JText::sprintf($this->params->get('params.filter_tip'), '<b>' . $this->label . '</b>', '<b>' . $date . '</b>') : NULL);
				
				switch ($this->params->get('params.filter_linkage'))
				{
					case 1 :
						$date = FilterHelper::filterLink('filter_' . $this->key, $value, $date, $this->type_id, $tip, $section);
						break;
					
					case 2 :
						$date .= ' ' . FilterHelper::filterButton('filter_' . $this->key, $value, $this->type_id, $tip, $section, $this->params->get('params.filter_icon', 'funnel-small.png'));
						break;
				}
			}
			$dates[] = $date;
		}
		
		$this->dates = $dates;
		
		return $this->_display_output($client, $record, $type, $section);
	} 

	public function _getFormatted($value, $only_format = false, $out = NULL)
	{
		if ($value == '')
			return '';
		if(is_array($value)) {
			return;
		}
		$params = $this->params;
		$format = $params->get('params.format' . $out, '%d %b %Y');
		if ($format == 'custom' && $params->get('params.custom' . $out, '') != '')
		{
			$format = $params->get('params.custom' . $out, '');
		}
		$date = new CDate($value);
		$date = $date->toFormat($format);
		if ($only_format)
			return $date;
		
		$diff = time() - strtotime($value);
		$days = $diff / 3600 / 24;
		
		if ($days < 0)
		{
			$type = 'normal';
			if (abs($days) <= $params->get('params.notify_days', 30))
			{
				$type = 'notify';
			}
		}
		else
		{
			$type = 'past';
		}
		$days = abs($days);
		switch ($params->get('params.computation', 'day'))
		{
			case 'round' :
				$days = round($days);
				break;
			case 'int' :
				$days = intval($days);
				break;
			case 'ceil' :
				$days = ceil($days);
				break;
			case 'day' :
				$days = intval($days) + 1;
				break;
		}
		
		$color = '';
		if ($params->get('params.' . $type . '_color') != '')
			$color = 'style="color: ' . $params->get('params.' . $type . '_color') . '"';
		
		switch ($params->get('params.mode', 2))
		{
			case '1' :
			case '3' :
				$out = JText::sprintf('%s %s %s', $params->get('params.' . $type . '_before'), $days, $params->get('params.' . $type . '_after'));
				if ($days == 1 && $type == 'notify')
				{
					$out .= ' (' . $params->get('params.notify_msg') . ')';
				}
				break;
			case '2' :
				$out = $date;
				break;
		}
		
		if ($params->get('params.mode', 2) == 3)
		{
			if ($params->get('params.show_days', 1) == 1)
				$out = $out . '<br>' . $date;
			else
				$out = $date . '<br>' . $out;
		}
		$out = JText::sprintf('<%s %s>%s</%s>', $params->get('params.' . $type . '_style'), $color, $out, $params->get('params.' . $type . '_style'));
		
		return $out;
	}
}	