<?php
/**
 * Cobalt by MintJoomla
 * a component for Joomla! 1.7 - 2.5 CMS (http://www.joomla.org)
 * Author Website: http://www.mintjoomla.com/
 * @copyright Copyright (C) 2012 MintJoomla (http://www.mintjoomla.com). All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */
defined('_JEXEC') or die();
require_once JPATH_ROOT . DS . 'components/com_cobalt/library/php/fields/cobaltfield.php';

class JFormFieldCGeo extends CFormField
{

	public function getInput()
	{
		$this->user = JFactory::getUser();

		return $this->_display_input();
	}

	public function validate($value, $record, $type, $section)
	{
		$links = self::getAditionalinks();
		$contacts = self::getAditionalFields();
		$address = self::getAddressFields();

		$this->_validate_group($value, 'links', $links);
		$this->_validate_group($value, 'contacts', $contacts);
		$this->_validate_group($value, 'address', $address);

		// check URLS to be entered correctly.
		if(isset($value['links']))
		{
			foreach($value['links'] as $name => $link)
			{
				if(! trim($link)) continue;
				if(trim($link) == 'http://') continue;
				if(! filter_var($link, FILTER_VALIDATE_URL))
				{
					$this->setError(JText::sprintf("G_URLINCORRECT", $links[$name]['label']));
				}
			}
		}

		if(isset($value['contacts']))
		{
			foreach($value['contacts'] as $name => $contact)
			{
				if(! trim($contact)) continue;

				if(! empty($contacts[$name]['filter']) && ! filter_var($contact, $contacts[$name]['filter']))
				{
					$this->setError(JText::sprintf("G_FORMATINCORRECT", $contacts[$name]['label']));
				}
				if(! empty($contacts[$name]['preg']) && ! preg_match('/^' . $contacts[$name]['preg'] . '$/', $contact))
				{
					$this->setError(JText::sprintf("G_FORMATINCORRECT", $contacts[$name]['label']));
				}
			}
		}

		if($this->params->get('params.map_marker'))
		{

		}

		parent::validate($value, $record, $type, $section);
	}

	public function onFilterWornLabel($section)
	{
		if(! $this->value) return;
		$value = $this->value;
		$user = JFactory::getUser();
		$out = null;
		$default = new JRegistry($value);
		if(in_array($this->params->get('params.filter_distance', 0), $user->getAuthorisedViewLevels()))
		{
			if($default->get('bounds.sw_lat', 0) && $default->get('bounds.ne_lat', 0))
			{
				$out = sprintf(JText::_("G_DISTANCEWORNLABEL"), $value['radius'], $value['dist_type'], $default->get('position.address'));
			}
		}
		if(in_array($this->params->get('params.filter_address', 0), $user->getAuthorisedViewLevels()) && $this->params->get('params.address.country.show'))
		{
			$addr = array();

			if(! empty($value['country'])) $addr[] = $value['country'];
			if(! empty($value['state'])) $addr[] = $value['state'];
			if(! empty($value['city'])) $addr[] = $value['city'];

			if($addr)
			{
				$out = implode('/', $addr);
			}
		}
		if(in_array($this->params->get('params.filter_marker', 0), $user->getAuthorisedViewLevels()))
		{
			if($default->get('marker.name'))
			{
				$out = '<img align="absmiddle" src="'.JURI::root().'components/com_cobalt/fields/geo/markers/'.$this->params->get('params.map_icon_src', 'custom').'/'.$default->get('marker.name').'" />';
				$out .= ' '.$this->_getMarkerName($default->get('marker.name'));
			}
		}
		return $out;
	}

	public function _getMarkerName($name)
	{
		$ext = JFile::getExt($name);
		$label = str_replace('.'.$ext, '', $name);
		$label = str_replace(array('-', '_'), '_', $label);
		$label = JText::_(ucwords($label));
		return $label;
	}
	public function onFilterWhere($section, &$query)
	{
		$user = JFactory::getUser();
		if(! $this->value) return;

		$default = new JRegistry($this->value);

		if(in_array($this->params->get('params.filter_distance', 0), $user->getAuthorisedViewLevels()))
		{
			if($default->get('bounds.sw_lat', 0) && $default->get('bounds.ne_lat', 0))
			{
				$records = array();
				$sql = "SELECT record_id FROM #__js_res_record_values WHERE value_index = 'lat' AND field_value BETWEEN ".$default->get('bounds.sw_lat', 0)." AND ".$default->get('bounds.ne_lat', 0);
				$ids = $this->getIds($sql);
				if($ids)
				{
					$sql = "SELECT record_id FROM #__js_res_record_values WHERE value_index = 'lng' AND field_value BETWEEN ".$default->get('bounds.sw_lng', 0)." AND ".$default->get('bounds.ne_lng', 0). " AND record_id IN (".implode(',', $ids).")";
					$records = $this->getIds($sql);
				}

				$records[] = 0;
				$query->where("r.id IN(".implode(',', $records).")");
				return TRUE;
			}
		}

		if(in_array($this->params->get('params.filter_address', 0), $user->getAuthorisedViewLevels()) && $this->params->get('params.address.country.show'))
		{
			if(! is_array($this->value)) return null;
			$this->section_id = $section->id;

			if($default->get('city'))
			{
				$inner = $this->_getCity($default->get('country'), $default->get('state'), $default->get('city'));
			}
			elseif ($default->get('state'))
			{
				$inner = $this->_getState($default->get('country'), $default->get('state'));
			}
			elseif($default->get('country'))
			{
				$inner = $this->_getCountry($default->get('country'));
			}

			if(!empty($inner))
			{
				$ids = $this->getIds($inner);

				$query->where("r.id IN(".implode(',', $ids).")");
				return TRUE;
			}
		}
		if(in_array($this->params->get('params.filter_marker', 0), $user->getAuthorisedViewLevels()))
		{
			if($default->get('marker.name'))
			{

				$sql = "SELECT record_id from #__js_res_record_values WHERE field_key = '{$this->key}' AND value_index = 'marker' AND field_value = '".$default->get('marker.name')."'";
				$ids = $this->getIds($sql);

				$query->where("r.id IN(".implode(',', $ids).")");
				return TRUE;
			}
		}
		return FALSE;
	}

	public function onRenderFilter($section, $module = false)
	{
		$this->user = JFactory::getUser();
		$this->markers = array();

		if (in_array($this->params->get('params.filter_marker', 0), $this->user->getAuthorisedViewLevels()))
		{
			$db = JFactory::getDbo();
			$sql = "SELECT DISTINCT field_value FROM #__js_res_record_values WHERE value_index = 'marker' AND field_key = '{$this->key}' AND section_id = {$section->id}";
			$db->setQuery($sql);
			$this->markers = $db->loadColumn();
		}

		return $this->_display_filter($section, $module);
	}

	public function onPrepareSave($value, $record, $type, $section)
	{
		return $value;
	}

	public function onStoreValues($record)
	{
		if(! empty($this->value['address']['country']))
		{
			$out['country'] = $this->value['address']['country'];
		}
		if(! empty($this->value['address']['zip']))
		{
			$out['zip'] = $this->value['address']['zip'];
		}
		if(! empty($this->value['address']['city']))
		{
			$out['city'] = $this->value['address']['city'];
		}
		if(! empty($this->value['address']['state']))
		{
			$out['state'] = $this->value['address']['state'];
		}
		if(! empty($this->value['position']['lat']))
		{
			$out['lat'] = $this->value['position']['lat'];
		}
		if(! empty($this->value['position']['lng']))
		{
			$out['lng'] = $this->value['position']['lng'];
		}
		if(! empty($this->value['position']['marker']))
		{
			$out['marker'] = $this->value['position']['marker'];
		}

		return $out;
	}

	public function onRenderFull($record, $type, $section)
	{
		return $this->_render(2, $record, $type, $section);
	}

	public function onRenderList($record, $type, $section)
	{
		return $this->_render(1, $record, $type, $section);
	}

	private function _render($client, $record, $type, $section)
	{
		static $data = array();

		$mecard = array();

		if(isset($data[$client . '-' . $record->id]))
		{
			return $data[$client . '-' . $record->id];
		}

		$this->user = JFactory::getUser();

		$email = null;
		if($this->params->get('params.qr_code_address'))
		{
			$db = JFactory::getDbo();
			$db->setQuery("SELECT field_value FROM #__js_res_record_values WHERE field_type = 'email' AND record_id = {$record->id}");
			$email = $db->loadResult();
		}
		$this->email = $email;

		$data[$client . '-' . $record->id] = $this->_display_output(($client == 1 ? 'list' : 'full'), $record, $type, $section);
		return $data[$client . '-' . $record->id];
	}

	public function _title($name, $client)
	{
		if(! in_array($this->params->get("params.adr_title"), array($client, 3)))
		{
			return;
		}

		return '<h4>' . $name . '</h4>';
	}

	public function _input($group, $name, $type = 'text')
	{
		return sprintf('<input class="inputbox" id="f%d_%s_%s" style="width:95%%" type="%s" name="jform[fields][%d][%s][%s]" value="%s" %s />', $this->id, $group, $name, $type, $this->id, $group, $name, (isset($this->value[$group][$name]) ? $this->value[$group][$name] : ($group == 'links' ? 'http://' : '')),
			($this->params->get("params.{$group}.{$name}.req") ? ' class="required" required="required"' : NULL));
	}

	public function _input_f($group, $name, $type = 'text')
	{
		return sprintf('<input id="f_%s_%s" style="width:100%%" type="%s" name="filters[%s][%s][%s]" value="%s" %s />', $group, $name, $type, $this->key, $group, $name, @$this->value[$group][$name], ($this->params->get("params.{$group}.{$name}.req") ? ' class="required" required="required"' : NULL));
	}

	public function _label($group, $name, $label)
	{
		$out[] = JText::_($label);

		if($this->params->get("params.{$group}.{$name}.req"))
		{
			$out[] = JHtml::image(JURI::root() . 'media/mint/icons/16/asterisk-small.png', 'Required');
		}

		return implode(' ', $out);
	}

	private function _validate_group($value, $group, $array)
	{
		foreach($array as $name => $val)
		{
			if(! $this->params->get("params.{$group}.{$name}.show"))
			{
				return;
			}

			if($this->params->get("params.{$group}.{$name}.req") && empty($value[$group][$name]))
			{
				$this->setError(JText::sprintf("CFIELDREQUIRED", $array[$name]['label']));
			}
		}
	}

	public function getAddressFields()
	{
		return array(
			'person' => array(
				'label' => JText::_('G_CONTACTPERSON'),
				'width' => 99,
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/person.png'
			),
			'address1' => array(
				'label' => JText::_('G_ADDRESS1'),
				'width' => 48
			),
			'address2' => array(
				'label' => JText::_('G_ADDRESS2'),
				'width' => 48
			),
			'city' => array(
				'label' => JText::_('G_CITY'),
				'width' => 48
			),
			'state' => array(
				'label' => JText::_('G_STATE'),
				'width' => 48
			),
			'zip' => array(
				'label' => JText::_('G_ZIP'),
				'width' => 48
			),
			'country' => array(
				'label' => JText::_('G_COUNTRY'),
				'width' => 48
			)
		);
	}

	public function getAditionalFields()
	{
		$out = array(
			'tel' => array(
				'label' => JText::_('G_TEL'),
				'patern' => '[NAME]: <a href="call://[VALUE]">[VALUE]</a>',
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/phone.png',
				'preg' => '[0-9\(\)\. \-\+\#]*'
			),
			'mob' => array(
				'label' => JText::_('G_MOBILE'),
				'patern' => '[NAME]: <a href="call://[VALUE]">[VALUE]</a>',
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/mobile.png',
				'preg' => '[0-9\(\)\. \+\#]*'
			),
			'fax' => array(
				'label' => JText::_('G_FAX'),
				'patern' => '[NAME]: <a href="call://[VALUE]">[VALUE]</a>',
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/fax.png',
				'preg' => '[0-9\(\)\. \-\+\#]*'
			),
			'icq' => array(
				'label' => JText::_('G_ICQ'),
				'patern' => '[NAME]: <a href="http://www.icq.com/people/[VALUE]#pager">[VALUE]</a>',
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/icq.png',
				'preg' => '[0-9]*'
			),
			'skype' => array(
				'label' => JText::_('G_SKYPE'),
				'patern' => '[NAME]: <a href="skype:[VALUE]?call">[VALUE]</a>',
				'icon' => 'http://mystatus.skype.com/smallicon/[VALUE]'
			),
			'google' => array(
				'label' => JText::_('G_GTALK'),
				'patern' => '[NAME]: [VALUE]',
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/google.png'
			),
			'y' => array(
				'label' => JText::_('G_YAHOOMSG'),
				'patern' => '[NAME]: <a href="ymsgr:sendim?[VALUE]">">[VALUE]</a>',
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/yahoo.png'
			),
			'msn' => array(
				'label' => JText::_('G_MSN'),
				'patern' => '[NAME]: <a href="msnim:chat?contact=[VALUE]">[VALUE]</a>',
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/msn.png',
				'filter' => FILTER_VALIDATE_EMAIL
			),
			'aol' => array(
				'label' => JText::_('G_AOL'),
				'patern' => '[NAME]: <a href="aim:goim?screenname=[VALUE]">[VALUE]</a>',
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/aol.png'
			)
		);

		if(JFile::exists(dirname(__FILE__) . DS . 'additional.php'))
		{
			include dirname(__FILE__) . DS . 'additional.php';
		}

		if(isset($contacts) && is_array($contacts))
		{
			$out = array_merge($out, $contacts);
		}
		return $out;

	}

	public function getAditionalinks()
	{
		$out = array(
			'web' => array(
				'label' => JText::_('G_SITE'),
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/web.png'
			),
			'facebook' => array(
				'label' => JText::_('G_FACEBOOK'),
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/fb.png'
			),
			'twitter' => array(
				'label' => JText::_('G_TWITTER'),
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/twitter.png'
			),
			'linkin' => array(
				'label' => JText::_('G_LINKIN'),
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/in.jpg'
			),
			'google' => array(
				'label' => JText::_('G_GOOGLEPLUS'),
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/google-plus.png'
			),
			'youtube' => array(
				'label' => JText::_('G_YOUTUBE'),
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/youtube.png'
			),
			'odnoclasnik' => array(
				'label' => JText::_('G_ODNOKLASSNIKI'),
				'icon' => JURI::root() . 'components/com_cobalt/fields/geo/icons/od.png'
			)
		);

		if(JFile::exists(dirname(__FILE__) . DS . 'additional.php'))
		{
			include dirname(__FILE__) . DS . 'additional.php';
		}

		if(isset($links) && is_array($links))
		{
			$out = array_merge($out, $links);
		}
		return $out;

	}

	public function _getChilds($post)
	{
		settype($this->value, 'array');
		$default = new JRegistry($this->value);

		$index = $post['index'];
		$list = $this->_getList($post);
		if($list)
		{
			$js = "onchange=\"getChilds(this.value, '{$post['child']}')\""; //($child ? "onchange=\"getChilds(this.value, '{$child}')\"" : NULL);
			return JHtml::_('select.genericlist', $list, "filters[{$this->key}][{$post['index']}]", $js, 'value', 'text', $default->get($post['index'], NULL));
		}
		return null;
	}

	private function _getList(&$post)
	{
		$index = $post['index'];
		$this->section_id = (int)$post['section_id'];

		$add = array();
		switch($index)
		{
			case 'country':
				$child = 'state';
				$country = null;
				$add[] = JHTML::_('select.option', 0, JText::_('G_SELECTCOUNTRY'), 'value', 'text');
				$query = $this->_getCountry();
				break;

			case 'state':
				$child = 'city';
				$country = $post['value'];
				$add[] = JHTML::_('select.option', 0, JText::_('G_SELECTSTATE'), 'value', 'text');
				$add[] = JHTML::_('select.option', "-1", JText::_('G_WITHOUTSTATE'), 'value', 'text');
				$query = $this->_getState($country);
				break;
			default:
				$child = null;
				$state = $post['value'];
				$country = $post['country'];
				$add[] = JHTML::_('select.option', 0, JText::_('G_SELECTCITY'), 'value', 'text');
				$query = $this->_getCity($country, $state);
		}
		$post['child'] = $child;

		$db = JFactory::getDbo();
		$db->setQuery($query);
		$list = $db->loadObjectList();

		if(count($list) <= 1 && $child)
		{
			$post['index'] = $child;
			$post['value'] = (isset($list[0]->value) ? $list[0]->value : NULL);
			$post['country'] = $country;
			$list = $this->_getList($post);
			$add = array();
		}

		if(count($list) == 0)
		{
			return array();
		}

		$list = array_merge($add, $list);
		return $list;
	}

	private function _getCountry($default = NULL)
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->from('#__js_res_record_values');
		$query->where("field_key = '{$this->key}'");
		$query->where("value_index = 'country'");
		$query->where("section_id = {$this->section_id}");

		if(! $default)
		{
			$query->select('field_value as value, field_value as text');
			$query->order('value');
			$query->group('value');
		}
		else
		{
			$query->select('record_id');
			$query->where("field_value = '{$default}'");
		}

		return $query;
	}

	private function _getState($country = NULL, $default = NULL)
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->from('#__js_res_record_values');
		$query->where("field_key = '{$this->key}'");
		$query->where("value_index = 'state'");
		$query->where("section_id = {$this->section_id}");

		if($country)
		{
			$query->where("record_id IN (" . $this->_getCountry($country) . ")");
		}

		if(! $default)
		{
			$query->select('field_value as value, field_value as text');
			$query->order('value');
			$query->group('value');
		}
		else
		{
			$query->select('record_id');
			$query->where("field_value = '{$default}'");
		}
		return $query;
	}

	private function _getCity($country, $state, $default = NULL)
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->from('#__js_res_record_values');
		$query->where("field_key = '{$this->key}'");
		$query->where("value_index = 'city'");
		$query->where("section_id = {$this->section_id}");

		if($state)
		{
			$query->where("record_id IN (" . $this->_getState($country, $state) . ")");
		}

		if($country)
		{
			$query->where("record_id IN (" . $this->_getCountry($country) . ")");
		}

		if(! $default)
		{
			$query->select('field_value as value, field_value as text');
			$query->order('value');
			$query->group('value');
		}
		else
		{
			$query->select('record_id');
			$query->where("field_value = '{$default}'");
		}
		return $query;
	}
	public function getMarker()
	{
		$icon_url = 'components/com_cobalt/fields/geo/markers/' . $this->params->get('params.map_icon', 'marker-big-red.png');
		if(!empty($this->value['position']['marker']))
		{
			$icon_url = 'components/com_cobalt/fields/geo/markers/'.$this->params->get('params.map_icon_src', 'custom').'/'.$this->value['position']['marker'];
		}

		$size = getimagesize(JPATH_ROOT.DS.$icon_url);

		$out = array($size[0], $size[1], JURI::root().$icon_url);

		return $out;
	}
}
