<?php
/**
 * Cobalt by MintJoomla
 * a component for Joomla! 1.7 - 2.5 CMS (http://www.joomla.org)
 * Author Website: http://www.mintjoomla.com/
 * @copyright Copyright (C) 2012 MintJoomla (http://www.mintjoomla.com). All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */
defined('_JEXEC') or die();
require_once JPATH_ROOT . DS . 'components/com_cobalt/library/php/fields/cobaltupload.php';
require_once JPATH_ROOT . DS . 'components/com_cobalt/library/php/fields/cobaltcommerce.php';
require_once JPATH_ROOT . DS . 'components/com_cobalt/library/php/commerce/mintpay.php';

class JFormFieldCPaytoaccess extends CFormField implements CFormFieldCommerce
{

	function __construct($field, $default)
	{
		parent::__construct($field, $default);
		
		$this->gateway = MintPay::getInstance($this->id, $this->params->get('params.default_gateway'), $this->params->get('params.log'));
		
		parent::__construct($field, $default);
		
		$this->subscriptions = array();
		if(isset($this->value['subscriptions']) && !empty($this->value['subscriptions']))
		{
			$this->subscriptions = $this->value['subscriptions'];
			unset($this->value['subscriptions']);
		}
	}

	public function getInput()
	{
		$this->pay = @$this->value['pay'];
		
		if($this->gateway) 
		{
			$this->gateway_form = $this->gateway->form($this);
		}
		else
		{
			$this->gateway_form = JText::sprintf('CGATEWAYNOTFOUND', '');
		}
		
		$user = JFactory::getUser();
		if($this->params->get('params.subscription', 0) && in_array($this->params->get('params.can_select_subscr', 0), $user->getAuthorisedViewLevels()))
		{
			$this->plans = JHtml::_('emerald.plans', "jform[fields][{$this->id}][subscriptions][]", $this->params->get('params.subscription', array()), $this->subscriptions, 'CRESTRICTIONPLANSACCESDESCR');
		}
		
		return $this->_display_input();
	}

	public function onReceivePayment($post, $record, &$controller)
	{
		if(! $this->gateway) return;
		$payment = $this->gateway->receive($this, $post, $record);
		$controller->setRedirect(JRoute::_(Url::record($record), FALSE));
	}

	private function _renderbutton()
	{
		if(! $this->gateway)
		{
			return;
		}
		$user = JFactory::getUser();
		ArrayHelper::clean_r($this->value);
		
		if(! $this->value)
		{
			return;
		}
		
		if(! $this->pay['amount'])
		{
			return;
		}
		
		$this->params->set('amount', $this->pay['amount']);
		$this->params->set('item_name', JText::sprintf($this->params->get('params.item_name', 'P_DOWNLOAD'), $this->record->title));
		
		$this->params->set('options', array());
		
		return $this->gateway->button($this);
	}

	public function onOrderList($order, $record)
	{
		return JText::_('CFPTAARTICLEUNLOCKED');
	}

	public function onStoreValues($record)
	{
		return array();
	}

	public function onRenderFull($record, $type, $section)
	{
		$this->_check($record);
		
		$app = JFactory::getApplication();
		
		if($errors = $this->getErrors())
		{
			foreach ($errors AS $err)
			{
				$app->enqueueMessage($err, 'error');
			}
			$url = JRoute::_(Url::records($section, JRequest::getInt('cat_id'), JRequest::getInt('user_id')), FALSE);
			$app->redirect($url);
			return;
		}
		
		$order = $this->gateway->get_order($this->user->get('id'), $record->id, $this->id);
		$this->order_table = $this->gateway->order_render('list', $order, $this);
		
		$subscr = $this->_ajast_subscr($record);
		ArrayHelper::clean_r($subscr);
		$this->subscr = $subscr;
		
		$this->ispaid = ((isset($order->status) && $order->status == 5) || ($subscr && $this->_is_subscribed(0)));
		
		return $this->_display_output('full', $record, $type, $section);
	}

	public function onRenderList($record, $type, $section)
	{
		$this->pay = @$this->value['pay'];
		$this->user = JFactory::getUser();
		$this->record = $record;
		$this->section = $section;
		
		$order = $this->gateway->get_order($this->user->get('id'), $record->id, $this->id);
		$this->order_table = $this->gateway->order_render('list', $order, $this);
		
		$subscr = $this->_ajast_subscr($record);
		ArrayHelper::clean_r($subscr);
		$this->subscr = $subscr;
		
		$this->ispaid = ((isset($order->status) && $order->status == 5) || ($subscr && $this->_is_subscribed(0)));
		
		$this->button = (object)$this->_renderbutton();
		$this->client = 'list';
		$this->order = $order;
		
		return $this->_display_output('list', $record, $type, $section);
	}

	protected function _check($record)
	{
		if(empty($this->value['pay']['amount']))
		{
			return;
		}
		
		$user = JFactory::getUser();
		
		if(! $user->get('id'))
		{
			$this->setError(JText::_('P_LOGINTOACCESSMSG'));
		}
		
		if($user->get('id') == $record->user_id)
		{
			return true;
		}
		
		if(in_array($this->params->get('params.skip_for'), $user->getAuthorisedViewLevels()))
		{
			return true;
		}
		
		$result = new stdClass();
		if($this->gateway) $result = $this->gateway->get_order($user->get('id'), $record->id, $this->id);
		
		if(empty($result->id))
		{
			$this->setError(JText::_('P_PURCHASETOACCESS'));
		}
		else
		{
			switch($result->status)
			{
				case 1:
					$this->setError(JText::_('P_PURCHASECANCELED'));
					break;
				case 2:
					$this->setError(JText::_('P_PURCHASEFAILED'));
					break;
				case 3:
					$this->setError(JText::_('P_PURCHASEPENDING'));
					break;
				case 4:
					$this->setError(JText::_('P_PURCHASEREFUNDED'));
					break;
			}
		}
		
		if($this->getErrors())
		{
			$subscr = $this->_ajast_subscr($record);
			ArrayHelper::clean_r($subscr);
			if($subscr)
			{
				$em_api = JPATH_ROOT . DS . 'components' . DS . 'com_emerald' . DS . 'api.php';
				if(!JFile::exists($em_api))
				{
					JError::raiseError(404, 'File API Emerald not found');
				}
				include_once ($em_api);
				if($this->_is_subscribed(0))
				{
					$this->error = array();
					EmeraldApi::addCount($this->params->get('params.subscription'), $this->params->get('params.subscription_count'));
				}
				else
				{
					$this->setError(JText::_('P_PURCHASETOACCESS2'));
				}
			}
		}
		
		if(! $this->getErrors() && $result->id)
		{
			$this->gateway->count($result->id);
		}
	}
	protected function _is_subscribed($redirect)
	{
		require_once JPATH_ROOT.DS.'components/com_emerald/api.php';
	
		return EmeraldApi::hasSubscription($this->params->get('params.subscription'),
		$this->params->get('params.subscription_msg'), $this->params->get('params.subscr_iid'),
		$redirect, $this->params->get('params.subscription_count'));
	}
	
	protected function _ajast_subscr($record)
	{
		static $ajasted = array();
	
		if(!$record->user_id) return;
	
		$user = JFactory::getUser($record->user_id);
	
		if(in_array($this->params->get('params.can_select_subscr', 0), $user->getAuthorisedViewLevels()) &&
		$this->params->get('params.subscription'))
		{
			return $this->subscriptions;
		}
		return $this->params->get('params.subscription');
	}
}
