<?php
/**
 * Cobalt by MintJoomla
 * a component for Joomla! 1.7 - 2.5 CMS (http://www.joomla.org)
 * Author Website: http://www.mintjoomla.com/
 * @copyright Copyright (C) 2012 MintJoomla (http://www.mintjoomla.com). All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */
defined('_JEXEC') or die();
require_once JPATH_ROOT . DS . 'components/com_cobalt/library/php/fields/cobaltupload.php';
require_once JPATH_ROOT . DS . 'components/com_cobalt/library/php/fields/cobaltcommerce.php';
require_once JPATH_ROOT . DS . 'components/com_cobalt/library/php/commerce/mintpay.php';

class JFormFieldCPaytodownload extends CFormFieldUpload implements CFormFieldCommerce
{

	function __construct($field, $default)
	{
		parent::__construct($field, $default);
		
		$this->gateway = MintPay::getInstance($this->id, $this->params->get('params.default_gateway'), $this->params->get('params.log'));
	}

	public function getInput()
	{
		/*
		 * if(!$this->gateway) { return JText::sprintf('CGATEWAYNOTFOUND', '');
		 * }
		 */
		$this->pay = @$this->value['pay'];
		$this->value = @$this->value['files'];
		
		if($this->gateway) $this->gateway_form = $this->gateway->form($this);
		else
			$this->gateway_form = JText::sprintf('CGATEWAYNOTFOUND', '');
		
		$params['width'] = $this->params->get('params.width', 0);
		$params['height'] = $this->params->get('params.height', 0);
		
		$params['max_size'] = ($this->params->get('params.max_size', 2000) * 1024);
		$params['method'] = $this->params->get('params.method', 'auto');
		$params['max_count'] = $this->params->get('params.max_count', 0);
		$params['file_formats'] = $this->params->get('params.file_formats', 'zip, jpg, png, gif, bmp');
		$params['allow_edit_title'] = $this->params->get('params.allow_edit_title', 1);
		$params['allow_add_descr'] = $this->params->get('params.allow_add_descr', 1);
		
		$this->options = $params;
		$this->fieldname = '[files]';
		
		$this->upload = parent::getInput();
		
		return $this->_display_input();
	}

	public function onReceivePayment($post, $record, &$controller, $request = null)
	{
		if(! $this->gateway) return;
		$payment = $this->gateway->receive($this, $post, $record);
		$controller->setRedirect(JRoute::_(Url::record($record), FALSE));
	}

	private function _renderbutton($client)
	{
		if(! $this->gateway)
		{
			return;
		}
		$user = JFactory::getUser();
		ArrayHelper::clean_r($this->value);
		
		if(! $this->value)
		{
			return;
		}
		
		if(! $this->pay['amount'])
		{
			return;
		}
		
		$this->params->set('amount', $this->pay['amount']);
		$this->params->set('item_name', JText::sprintf($this->params->get('params.item_name', 'P_DOWNLOAD'), $this->record->title));
		
		$options = array();
		foreach($this->value as $k => $file)
		{
			$options[JText::_('CFILE') . ' ' . ($k + 1)] = $file['realname'];
		}
		$this->params->set('options', $options);
		
		return $this->gateway->button($this);
	}

	public function onBeforeDownload($record, $file_index, $file_id)
	{
		if(empty($this->value['pay']['amount']))
		{
			return;
		}
		
		$user = JFactory::getUser();
		
		if(! $user->get('id'))
		{
			$this->setError(JText::_('P_LOGINTODOWNLOADMSG'));
		}
		
		if($user->get('id') == $record->user_id)
		{
			return true;
		}
		
		if(in_array($this->params->get('params.skip_for'), $user->getAuthorisedViewLevels()))
		{
			return true;
		}
		
		$result = new stdClass();
		if($this->gateway) $result = $this->gateway->get_order($user->get('id'), $record->id, $this->id);
		
		if(empty($result->id))
		{
			$this->setError(JText::_('P_PURCHASETODOWNLOAD'));
		}
		else 
		{
			switch($result->status)
			{
				case 1:
					$this->setError(JText::_('P_PURCHASECANCELED'));
				break;
				case 2:
					$this->setError(JText::_('P_PURCHASEFAILED'));
				break;
				case 3:
					$this->setError(JText::_('P_PURCHASEPENDING'));
				break;
				case 4:
					$this->setError(JText::_('P_PURCHASEREFUNDED'));
				break;
			}
		}
		
		if($this->getErrors())
		{
			if(parent::onBeforeDownload($record, $file_index, $file_id))
			{
				$this->error = array();
			}
		}
		
		if(! $this->getErrors() && $result->id)
		{
			MintPay::getInstance($this->id, $this->params->get('params.default_gateway'), 0)->count($result->id);
		}
	}

	public function onOrderList($order, $record)
	{
		$section = ItemsStore::getSection($record->section_id);
		$type = ItemsStore::getType($record->type_id);
		$out = $this->_render(1, 0, $record, $type, $section);
		
		return $out;
	}

	public function onPrepareSave($value, $record, $type, $section)
	{
		if(isset($value['files']))
		{
			$value['files'] = $this->_getPrepared($value['files']);
		}
		return $value;
	}

	public function onStoreValues($record)
	{
		$value = @$this->value['files'];
		settype($value, 'array');
		$out = $saved = array();
		foreach($value as $file)
		{
			$out[] = $file['realname'];
			$saved[] = $file['id'];
		}
		
		$files = JTable::getInstance('Files', 'CobaltTable');
		$files->markSaved($saved, $record, $this->id);
		
		return $out;
	}

	public function onCopy($value, $record, $type, $section)
	{
		if(! empty($value))
		{
			foreach($value['files'] as $key => $file)
			{
				
				$value['files'][$key] = $this->copyFile($file);
			}
		}
		return $value;
	}

	public function onRenderFull($record, $type, $section)
	{
		return $this->_render(2, 0, $record, $type, $section);
	}

	public function onRenderList($record, $type, $section)
	{
		return $this->_render(1, $this->params->get('params.list_limit', 5),$record, $type, $section);
	}

	private function _render($client, $limit = 0, $record, $type, $section)
	{
		$this->pay = @$this->value['pay'];
		$this->value = @$this->value['files'];
		$this->user = JFactory::getUser();
		$this->record = $record;
		$this->section = $section;
		
		$hits = in_array($this->params->get('params.show_hit'), array(3, $client ));
		$files = $this->getFiles($record, $hits);
		
		if($limit)
		{
			$files = array_slice($files, 0, $limit);
		}
		
		if(! $files) return;
		
		
		$order = $this->gateway->get_order($this->user->get('id'), $record->id, $this->id);
		$this->order_table = $this->gateway->order_render($client, $order, $this);
		
		$subscr = $this->_ajast_subscr($record);
		ArrayHelper::clean_r($subscr);
		$this->subscr = $subscr;
		
		$this->ispaid = ((isset($order->status) && $order->status == 5) || ($subscr && $this->_is_subscribed(0)));	
		
		$this->button = (object)$this->_renderbutton($client == 1 ? 'list' : 'full');
		$this->client = $client;
		$this->files = $files;
		$this->order = $order;
		
		
		
		return $this->_display_output(($client == 1 ? 'list' : 'full'), $record, $type, $section);
	}
	
	/* private function _render($hits, $format, $limit = 0)
	{
		$this->pay = @$this->value['pay'];
		$this->value = @$this->value['files'];
		$user = JFactory::getUser();
		
		$files = $this->getFiles($this->record, $hits);
		$el = array();
		
		foreach($files as $k => $file)
		{
			if($limit && ($limit <= $k))
			{
				break;
			}
			if(is_array($file))
			{
				$file = JArrayHelper::toObject($file);
			}
			$h = ($hits ? sprintf(' <b>%s</b>: <span style="color:purple">%d</span>', JText::_('CHITS'), $file->hits) : NULL);
			$s = ($this->params->get('params.show_size') ? sprintf(' <b>%s</b>: <span style="color:green">%s</span>', JText::_('CSIZE'), HTMLFormatHelper::formatSize($file->size)) : NULL);
			$el[] = sprintf('<a href="%s">%s</a> %s %s', $file->url, $file->realname, $h, $s);
		}
		
		if(! $el) return;
		
		switch($format)
		{
			case 1:
				$out = implode(', ', $el);
			break;
			
			case 2:
				$out = "<ul style=\"display:inline-block\"><li>" . implode("</li><li>", $el) . "</li></ul>";
			break;
		}
		
		return $out;
	} */
}
