<?php
/**
 * Cobalt by MintJoomla
 * a component for Joomla! 1.7 - 2.5 CMS (http://www.joomla.org)
 * Author Website: http://www.mintjoomla.com/
 * @copyright Copyright (C) 2012 MintJoomla (http://www.mintjoomla.com). All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */

defined('_JEXEC') or die();
require_once JPATH_ROOT . DS . 'components/com_cobalt/library/php/fields/cobaltfield.php';

class CFormFieldSelectable extends CFormField
{

	public function onJSValidate()
	{
	}

	public function validate($value, $record, $type, $section)
	{
		return parent::validate($value, $record, $type, $section);
	}

	public function onPrepareFullTextSearch($value, $record, $type, $section)
	{
		ArrayHelper::clean_r($value);
		ArrayHelper::separate_r($value, $this->params->get('params.color_separator', '^'));
		return implode(', ', $value);
	}

	public function onPrepareSave($value, $record, $type, $section)
	{
		if(!$value) return;
		if ($this->type == 'listautocomplete')
		{
			$value = explode(',', $value);
		}
		if ($this->params->get('params.sql_source'))
		{
			$db = JFactory::getDbo();
			$db->setQuery($this->params->get('params.sql', "SELECT 1 AS value, 'No sql query entered' AS text"));
			$values = $db->loadAssocList('id');
			
			settype($value, 'array');
			foreach($value as &$v)
			{
				$out[$v] = $values[$v]['text'];
			}
			$value = $out;
		}
		else
		{
			ArrayHelper::clean_r($value);
			if ($this->params->get('params.total_limit'))
			{
				$value = array_chunk($value, $this->params->get('params.total_limit'));
				$value = $value[0];
				ArrayHelper::clean_r($value);
			}
		}
		return $value;
	}

	public function onStoreValues($record)
	{
		if ($this->params->get('params.save_new', 1) && isset($this->value))
		{
			$list = explode("\n", str_replace("\r", "", $this->params->get('params.values', '')));
			$list = $list2 = array_values($list);
			ArrayHelper::clean_r($list);
			ArrayHelper::tolower_r($list2);
			ArrayHelper::separate_r($list2, $this->params->get('params.color_separator', '^'));
			
			$save = FALSE;
			foreach($this->value as $value)
			{
				$val = explode($this->params->get('params.color_separator', '^'), $value);
				if (!in_array(JString::strtolower($val[0]), $list2))
				{
					$list[] = $value;
					$save = TRUE;
				}
			}
			ArrayHelper::clean_r($list);
			
			if ($save)
			{
				$this->params->set('params.values', implode("\n", $list));
				$params = $this->params->toString();
				
				$table = JTable::getInstance('Field', 'Cobalttable');
				$table->load($this->id);
				$table->params = $params;
				$table->store();
			}
		}
		return $this->value;
	}

	public function onRenderFilter($section, $module = false)
	{
		$db = JFactory::getDbo();
		
		$query = $db->getQuery(TRUE);
		
		$query->select('field_value');
		$query->from('#__js_res_record_values');
		$query->where("section_id = {$section->id}");
		$query->where("`field_key` = '{$this->key}'");
		$query->group('field_value');
		
		if ($this->params->get('params.filter_show_number', 1))
		{
			$query->select('count(record_id) as num');
		}
		$db->setQuery($query);
		
		$this->values = $db->loadObjectList();
		
		if(!$this->values) return;
		
		
		return $this->_display_filter($section, $module);		
		
	}

	public function onFilterWornLabel($section)
	{
		$value = $this->value;
		$label = array();
		settype($value, 'array');
		foreach($value as $val)
		{
			$c = explode($this->params->get('params.color_separator', '^'), $val);
			ArrayHelper::clean_r($c);
			$label[] = (isset($c[1]) ? "<SPAN style=\"color:{$c[1]}\">".JText::_($c[0])."</SPAN>" : JText::_($c[0]));
		}
		$value = implode(', ', $label);
		return $value;
	}

	public function onFilterWhere($section, &$query)
	{
		$value = $this->value;
		if($this->type == 'listautocomplete' && !is_array($value))
		{
			$value = explode(',', $value);
		}		
		ArrayHelper::trim_r($value);
		ArrayHelper::clean_r($value);
		
		if (!$value)
			return NULL;
		
		$db = JFactory::getDbo();
		
		foreach($value as $text)
		{
			$sql[] = "field_value LIKE '" . $db->escape($text) . "%'";
		}
		$db = JFactory::getDbo();
		$db->setQuery("SELECT record_id FROM #__js_res_record_values WHERE (" . implode(' OR ', $sql) . ") AND section_id = {$section->id} AND field_key = '{$this->key}'");
		$ids = $db->loadColumn(); 
		$ids[] = 0;
		
		$query->where("r.id IN(".implode(',', $ids).")");
		return true;
	}

	public function onRenderFull($record, $type, $section)
	{
		$this->record = $record;
		return $this->_getvalues(0, $this->params->get('params.list_type_record', 1),$record, $type, $section, 'full');
	}

	public function onRenderList($record, $type, $section)
	{
		$this->record = $record;
		return $this->_getvalues($this->params->get('params.list_limit', 5), $this->params->get('params.list_type_list', 1),$record, $type, $section, 'list');
	}

	private function _getvalues($limit = 0, $list_type = 1, $record, $type, $section, $client)
	{
		$user = JFactory::getUser();
		if ($this->type == 'listautocomplete')
		{
			if(is_string($this->value))
			$this->value = explode(',', $this->value);
		}
		ArrayHelper::clean_r($this->value);
		$values = $i = null;
		foreach($this->value as $k => $value)
		{
			if ($limit > 0 && ($i >= $limit))
			{
				break;
			}
			$i ++;
			
			$c = explode($this->params->get('params.color_separator', '^'), $value);
			ArrayHelper::clean_r($c);
			
			$text = JText::_($c[0]);
			if (isset($c[1]))
			{
				$text = "<b style=\"color:{$c[1]}\">{$text}</b>";
			}
			
			if ($this->params->get('params.sql_link') && $this->params->get('params.sql_source'))
			{
				$attributes = ($this->params->get('params.sql_link_target') ? array('target' => '_blank') : array());
				$process = str_replace(array('[ID]', '[USER_ID]', '[AUTHOR_ID]'), array($k, $user->get('id', 0), $this->record->user_id), $this->params->get('params.sql_link'));
				if(substr($process, 0, 9) == 'index.php')
				{
					$process = JRoute::_($process); 
				}
				$text = JHtml::link($process, $text, $attributes);
				$this->params->set('params.filter_linkage', 2);
			}
			
			if ($this->params->get('params.filter_enable'))
			{
				$tip = ($this->params->get('params.filter_tip') ? JText::sprintf($this->params->get('params.filter_tip'), '<b>' . $this->label . '</b>', $text) : NULL);
				switch ($this->params->get('params.filter_linkage'))
				{
					case 1 :
						$value = FilterHelper::filterLink('filter_' . $this->key, urlencode($value), $text, $this->type_id, $tip, $section);
						break;
					
					case 2 :
						$value = $text . ' ' . FilterHelper::filterButton('filter_' . $this->key, urlencode($value), $this->type_id, $tip, $section, $this->params->get('params.filter_icon', 'funnel-small.png'));
						break;
				}
			}
			else
			{
				$value = $text;
			}
			
			$values[] = $value;
		}
		if (!$values)
		{
			return null;
		}
		$this->values = $values;
		$this->listtype = $list_type;
		
		return $this->_display_output($client, $record, $type, $section);
	}

	protected function _custom_value($type)
	{
		$out = array();
		JFactory::getDocument()->addScript(JURI::root() . 'media/mint/js/GrowingInput.js');
		$out[] = '<div style="clear:both"></div>';
		$out[] = sprintf('<div class="variant-container" id="variant_%d">
			<a class="small" id="show_variant_link_%d" rel="{field_type:\'%s\', id:%d, inputtype:\'%s\', width:%d, limit:%d, max_size:%d}" 
				href="javascript:void(0)" onclick="showAddForm(this)">%s</a></div>', $this->id, $this->id, $this->type, $this->id, $type, $this->params->get('params.width', '220'), $this->params->get('params.total_limit', 0), $this->params->get('params.size', 0), JText::_($this->params->get('params.user_value_label', 'Your variant')));
		$out[] = '<div style="clear:both"></div>';
		
		return implode("\n", $out);
	}

	private function _input_user_value()
	{
	
	}

}
