<?php
/**
 * Cobalt by MintJoomla
 * a component for Joomla! 1.7 - 2.5 CMS (http://www.joomla.org)
 * Author Website: http://www.mintjoomla.com/
 * @copyright Copyright (C) 2012 MintJoomla (http://www.mintjoomla.com). All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */

defined('_JEXEC') or die();
require_once JPATH_ROOT.DS.'components/com_cobalt/library/php/fields/cobaltfield.php';

class CFormFieldUpload extends CFormField
{

	public function __construct($field, $default)
	{
		$root = JPath::clean(JComponentHelper::getParams('com_cobalt')->get('general_upload'));
		$url = str_replace(JPATH_ROOT, '', $root);
		$url = str_replace("\\", '/', $url);
		$url = preg_replace('#^\/#iU', '', $url);
		$this->url = JURI::root() . str_replace("//", "/", $url);

		$this->fieldname = NULL;

		parent::__construct($field, $default);

		$this->subscriptions = array();
		if(isset($this->value['subscriptions']) && !empty($this->value['subscriptions']))
		{
			$this->subscriptions = $this->value['subscriptions'];
			unset($this->value['subscriptions']);
		}
	}

	protected function getFileUrl($file)
	{
		$out = $this->url . "/{$file->subfolder}/" . $file->fullpath;

		return $out;
	}

	public function onJSValidate()
	{
		$js = "$('adminForm').getElements('input[id^=\"".$this->tmpname."_tbxFile\"]').destroy();";
		return $js;
	}

	public function getInput()
	{
		$user = JFactory::getUser();
		settype($this->value, 'array');
		$default = array();
		if(isset($this->value[0]))
		{
			if(is_array($this->value[0]))
			{
				$default = $this->value;
			}
			else
			{
				$default = $this->getFiles($this->record);
			}
		}

		$this->options['can_delete'] = $this->_getDeleteAccess();
		$this->tmpname = $this->options['tmpname'] = substr(md5(time() . rand(1, 1000000)), 0, 5);

		$html = JHtml::_('mrelements.mooupload', "jform[fields][{$this->id}]".$this->fieldname, $default, $this->options , $this->id);

		if($this->params->get('params.subscription', 0) && in_array($this->params->get('params.can_select_subscr', 0), $user->getAuthorisedViewLevels()))
		{
			$html .= JHtml::_('emerald.plans', "jform[fields][{$this->id}][subscriptions][]", $this->params->get('params.subscription', array()), $this->subscriptions, 'CRESTRICTIONPLANSDESCR');
		}
		return $html;
	}

	private function _getDeleteAccess()
	{
		$user = JFactory::getUser();
		$author_can_delete = $this->params->get('params.delete_access', 1);
		$params = JComponentHelper::getParams('com_cobalt');
		$type = ItemsStore::getType($this->type_id);

		$record_id = JRequest::getInt('id', 0);

		if($author_can_delete && (!$record_id || $user->get('id') == ItemsStore::getRecord($record_id)->user_id))
		{
			return 1;
		}
		else
		{
			if($params->get('moderator') == $user->get('id'))
			{
				return 1;
			}

			if(in_array($type->params->get('properties.item_can_moderate'), $user->getAuthorisedViewLevels()))
			{
				return 1;
			}

			if(MECAccess::allowUserModerate($user, ItemsStore::getSection(JRequest::getInt('section_id')), 'allow_delete'))
			{
				return 1;
			}
		}

		return 0;
	}

	public function onPrepareSave($value, $record, $type, $section)
	{
		$subscr = false;

		if(isset($value['subscriptions']))
		{
			$subscr = $value['subscriptions'];
			unset($value['subscriptions']);
		}

		$result = $this->_getPrepared($value);

		if($subscr)
		{
			$result['subscriptions'] = $subscr;
		}

		return $result;
	}

	public function onPrepareFullTextSearch($value, $record, $type, $section)
	{
		$files = $this->_getPrepared($value);

		$out = array();
		foreach($files as $file)
		{
			$out[] = $file->realname;
		}

		return implode(', ', $out);
	}

	public function onStoreValues($record)
	{
		settype($this->value, 'array');
		$out = $saved = array();
		foreach($this->value as $key => $file)
		{
			if(!JString::strcmp($key, 'subscriptions')) continue;
			$out[] = $file['realname'];
			$saved[] = $file['id'];
		}

		$files = JTable::getInstance('Files', 'CobaltTable');
		$files->markSaved($saved, $record, $this->id);

		return $out;
	}

	protected function _getPrepared($array)
	{
		static $data = array();

		settype($array, 'array');

		$key = md5(implode(',', $array));

		if(isset($data[$key]))
		{
			return $data[$key];
		}
		$files = JTable::getInstance('Files', 'CobaltTable');
		$array = $files->prepareSave($array);
		$data[$key] = json_decode($array);

		return $data[$key];

	}

	public function onBeforeDownload($record, $file_index, $file_id)
	{
		$user = JFactory::getUser();
		if(!in_array($this->params->get('params.allow_download', 1), $user->getAuthorisedViewLevels()))
		{
			$this->setError(JText::_("CNORIGHTSDOWNLOAD"));
			return FALSE;
		}

		if($this->_ajast_subscr($record))
		{
			$em_api = JPATH_ROOT . DS . 'components' . DS . 'com_emerald' . DS . 'api.php';
			if(!JFile::exists($em_api))
			{
				return true;
			}

			if(in_array($this->params->get('params.subscr_skip', 3), $user->getAuthorisedViewLevels()))
			{
				return true;
			}

			if($this->params->get('params.subscr_author_skip', 1) && $record->user_id && ($record->user_id == $user->id))
			{
				return true;
			}
			$section = JModel::getInstance('Section', 'CobaltModel')->getItem($record->section_id);
			if($this->params->get('params.subscr_moderator_skip', 1) && MECAccess::allowRestricted($user, $section))
			{
				return true;
			}

			include_once ($em_api);
			if($this->_is_subscribed($this->params->get('params.subscription_redirect')))
			{
				EmeraldApi::addCount($this->params->get('params.subscription'), $this->params->get('params.subscription_count'));
				return true;
			}
			$plans = $this->params->get('params.subscription');
			settype($plans, 'array');
			$url = 'index.php?option=com_emerald&view=plans&layout=form&Itemid=' . $this->params->get('params.subscr_iid') . '&id=' . implode(',', $plans);

			$this->setError(JText::_($this->params->get('params.subscription_msg')).' '.
				JHtml::link(JRoute::_($url), JText::_('CSUBSCRIBE')));
			return FALSE;
		}
		return FALSE;
	}

	protected function _is_subscribed($redirect)
	{
		require_once JPATH_ROOT.DS.'components/com_emerald/api.php';

		return EmeraldApi::hasSubscription($this->params->get('params.subscription'),
				$this->params->get('params.subscription_msg'), $this->params->get('params.subscr_iid'),
				$redirect, $this->params->get('params.subscription_count'));
	}

	protected function _ajast_subscr($record)
	{
		static $ajasted = array();

		if(!$record->user_id) return;

		$user = JFactory::getUser($record->user_id);

		if(in_array($this->params->get('params.can_select_subscr', 0), $user->getAuthorisedViewLevels()) &&
		$this->params->get('params.subscription'))
		{
			return $this->subscriptions;
		}
		return $this->params->get('params.subscription');
	}

	public function onCopy($value, $record, $type, $section)
	{
		if(!empty($value))
		{
			foreach ($value AS $key => $file)
			{
				$value[$key] = $this->copyFile($file);
			}
		}
		return $value;
	}

	protected function getFiles($record, $show_hits = FALSE)
	{
		$list = $this->value;
		$subfolder = $this->params->get('params.subfolder' , false);

		if(! $list)
		{
			return array();
		}

		if(is_string($list))
		{
			$list = json_decode($list);
		}

		$files = JTable::getInstance('Files', 'CobaltTable');
		if(!is_array($list[0]))
		{
			$list = $files->getFiles($list, 'filename');
			$show_hits = false;
		}

		if($show_hits)
		{
			$in = array();
			foreach($list as $attach)
			{
				settype($attach, 'array');
				$in[] = $attach['id'];
			}

			if($in)
			{
// 				var_dump($in);
				$list = $files->getFiles($in);
			}
		}
		foreach($list as $idx => &$file)
		{

			//var_dump($file);
			if(is_array($file))
			{
				$file = JArrayHelper::toObject($file);
			}
			if($this->params->get('params.show_in_browser', 0) == 0)
			{
				$file->url = $this->getDownloadUrl($record, $file, $idx);
			}
			else
			{
				$file->url = JURI::root().JComponentHelper::getParams('com_cobalt')->get('general_upload').'/'.$subfolder.'/'.str_replace('\\', '/', $file->fullpath);
			}
			$file->subfolder = $subfolder ? $subfolder : $file->ext;
		}

		$sort = $this->params->get('params.sort', 0);

		$parts = explode(' ', $sort);
		if(!isset($parts[0]))
		{
			$parts[0] = 0;
		}

		if(!isset($parts[1]))
		{
			$parts[1] = 'ASC';
		}

		switch($parts[0])
		{
			case 0:
				$title = $this->params->get('params.allow_edit_title', 0);
				foreach ($list as $val)
				{
					$sortArray[] = strtolower($title && $val->title ? $val->title : $val->realname);
				}
 				natcasesort($sortArray);
				array_multisort($sortArray, constant('SORT_'.$parts[1]), $list);
				break;

			case 1:
				foreach ($list as $val)
				{
					$sortArray[] = $val->size;
				}
				array_multisort($sortArray, constant('SORT_'.$parts[1]), $list);
				break;

			case 2:
				foreach ($list as $val)
				{
					$sortArray[] = $val->hits;
				}
				array_multisort($sortArray, constant('SORT_'.$parts[1]), $list);
				break;
		}


		return $list;
	}

	protected function getDownloadUrl($record, $file, $idx)
	{
		if(empty($record)) return;
		$url = JURI::root() . 'index.php?option=com_cobalt&task=files.download&no_html=1';
		$url .= '&id=' . $file->id;
		$url .= '&fid=' . $this->id;
		$url .= '&fidx=' . $idx;
		$url .= '&rid=' . $record->id;
		$url .= '&return=' . urlencode(base64_encode(JFactory::getURI()->toString()));

		return $url;
	}

	/**
	 *
	 * @param string $filename Value from column 'filename' in table #__js_res_files
	 * @return string Filename of copied file
	 */

	protected function copyFile($filename)
	{
		$params = JComponentHelper::getParams('com_cobalt');
		$files_table = JTable::getInstance('Files', 'CobaltTable');
		if($files_table->load(array('filename' => $filename)))
		{
			$time = time();
			$date = date('Y-m', $time);
			$ext = strtolower(JFile::getExt($filename));
			$dest = JPATH_ROOT.DS.$params->get('general_upload').DS.$ext.DS.$date.DS;

			if(!JFolder::exists($dest))
			{
				JFolder::create($dest, 0777);
				JFile::write($dest.DS.'index.html', $index = '<html><body></body></html>');
			}
			$files_table->id = null;
			$parts = explode('_', $filename);
			$files_table->filename = $time.'_'.$parts[1];

			$copied = JFile::copy(JPATH_ROOT.DS.$params->get('general_upload').DS.$files_table->fullpath, $dest.$files_table->filename);

			$root = JPath::clean(JPATH_ROOT.DS.$params->get('general_upload'));
			$url = str_replace(JPATH_ROOT, '', $root);
			$url = str_replace("\\", '/', $url);
			$files_table->url = JURI::root().str_replace("//", "/", $url.'/'.$ext.'/'.$date.'/'.$files_table->filename);
			$files_table->fullpath = JPath::clean($ext.DS.$date.DS.$files_table->filename);
			//$metadata = exif_read_data($files_table->fullpath);
			//$files_table->params = json_encode($metadata);
			$files_table->saved = 0;

			if(!$copied)
			{
				return false;
			}
			if(!$files_table->store())
			{
				return false;
			}
			return $files_table->filename;
		}

		return false;
	}

	public function onSaveTitle()
	{
		$id = JRequest::getVar('id', 0);
		$text = CensorHelper::cleanText(JRequest::getVar('text'));
		$record_id = JRequest::getVar('record_id', 0);
		$field_id = JRequest::getVar('field_id', 0);
		if($record_id && $field_id)
		{
			$record_table = JTable::getInstance('Record', 'CobaltTable');
			$record_table->load($record_id);
			$fields = json_decode($record_table->fields, true);

			if(isset($fields[$field_id]))
			{
				$files = &$fields[$field_id];
				if(isset($fields[$field_id]['files']))
				{
					$files = &$fields[$field_id]['files'];
				}

				foreach ($files as &$file)
				{
					if($file['id'] == $id)
					{
						$file['title'] = $text;
						break;
					}
				}
				$record_table->fields = json_encode($fields);
				$record_table->store();
			}

		}
		$db = JFactory::getDbo();
		$db->setQuery("UPDATE #__js_res_files SET title = '{$text}' WHERE id = {$id}");
		if($db->query())
		{
			echo json_encode(array('success' => '1', 'result' => $text));
		}
		else
		{
			echo json_encode(array('success' => '0'));
		}
	}

	public function onSaveDescr()
	{
		$id = JRequest::getVar('id', 0);
		$text = CensorHelper::cleanText(JRequest::getVar('text', $default = null, $hash = 'default', $type = 'none', $mask = 4));
		$record_id = JRequest::getVar('record_id', 0);
		$field_id = JRequest::getVar('field_id', 0);
		if($record_id && $field_id)
		{
			$record_table = JTable::getInstance('Record', 'CobaltTable');
			$record_table->load($record_id);
			$fields = json_decode($record_table->fields, true);

			if(isset($fields[$field_id]))
			{
				$files = &$fields[$field_id];
				if(isset($fields[$field_id]['files']))
				{
					$files = &$fields[$field_id]['files'];
				}

				foreach ($files as &$file)
				{
					if($file['id'] == $id)
					{
						$file['description'] = $text;
						break;
					}
				}
				$record_table->fields = json_encode($fields);
				$record_table->store();
			}

		}
		$db = JFactory::getDbo();
		$db->setQuery("UPDATE #__js_res_files SET description = '{$text}' WHERE id = {$id}");
		if($db->query())
		{
			echo json_encode(array('success' => '1', 'result' => $text));
		}
		else
		{
			echo json_encode(array('success' => '0'));
		}
	}
}
