<?php
/**
 * Cobalt by MintJoomla
 * a component for Joomla! 1.7 - 2.5 CMS (http://www.joomla.org)
 * Author Website: http://www.mintjoomla.com/
 * @copyright Copyright (C) 2012 MintJoomla (http://www.mintjoomla.com). All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */
defined('_JEXEC') or die;

// Base this model on the backend version.
//require_once JPATH_ADMINISTRATOR.'/components/com_content/models/article.php';

jimport('joomla.application.component.modelform');
jimport('joomla.application.component.modeladmin');

class CobaltModelForm extends JModelAdmin
{
	public function getForm($data = array(), $loadData = true)
	{
		$form = $this->loadForm('com_cobalt.record', 'record', array('control' => 'jform', 'load_data' => $loadData));
		if(empty($form))
		{
			return false;
		}
		return $form;
	}

	protected function loadFormData()
	{
		$data = JFactory::getApplication()->getUserState('com_cobalt.edit.form.data', array());
		if(empty($data))
		{
			$data = $this->getItem();
		}

		return $data;
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$app = JApplication::getInstance('Site');

		// Load state from the request.
		$pk = JRequest::getInt('id');
		JFactory::getApplication()->setUserState('com_cobalt.edit.form.id',  $pk);
		$this->setState('com_cobalt.edit.form.id', $pk);

		$return = JRequest::getVar('return', null, 'default', 'base64');
		$this->setState('return_page', base64_decode($return));

		// Load the parameters.
		$params	= $app->getParams();
		$this->setState('params', $params);

		$this->setState('layout', JRequest::getCmd('layout'));
	}

	public function getTable($name = '', $prefix = 'Table', $options = array()){
		return JTable::getInstance('Record', 'CobaltTable');
	}

	public function getItem($itemId = null, $cache = TRUE)
	{
		static $data = array();

		$itemId = (int) ($itemId ? $itemId : $this->getState('com_cobalt.edit.form.id'));

		if(!$itemId)
		{

			$type = $this->getRecordType(JRequest::getInt('type_id'), $itemId);

			$user = JFactory::getUser();
			$out = new stdClass();
			$out->id = NULL;
			$out->type_id = JRequest::getInt('type_id');
			$out->section_id = JRequest::getInt('section_id');
			$out->published = $type->params->get('submission.autopublish', 1);
			$out->ctime = JFactory::getDate()->toSql();
			$out->mtime = JFactory::getDate()->toSql();
			$out->access = $type->params->get('submission.access', 1);
			$out->fields = '[{}]';
			$out->categories = array();
			$out->user_id = $user->get('id');

			if($type->params->get('submission.default_expire', 0) > 0)
			{
				$out->extime = JFactory::getDate('+ '.$type->params->get('submission.default_expire').' DAY')->toSql();
			}

			return $out;
		}


		if(isset($data[$itemId]) && $cache)
		{
			return $data[$itemId];
		}

		$table = $this->getTable();
		$return = $table->load($itemId);

		if ($return === false && $table->getError()) {
			$this->setError($table->getError());
			return false;
		}

		JRequest::setVar('type_id', $table->type_id);
		JRequest::setVar('section_id', $table->section_id);

		$params = new JRegistry();
		if(!empty($table->params))
		{
			$params->loadString($table->params);
		}
		$table->params = $params;

		$table->categories = json_decode($table->categories, true);

		$data[$itemId] = $table;

		return $data[$itemId];

	}
	public function getRecordType($typeId = NULL, $itemId = NULL)
	{

		static $data = array();


		if(!$typeId)
		{
			$typeId = JRequest::getInt('type_id');
		}

		if(!$typeId && $itemId)
		{
			$item = $this->getItem();
			$typeId = $item->type_id;
		}

		if(!$typeId)
		{
			$this->setError('Type not set');
			$out = new stdClass();
			$out->params = new JRegistry();
			return $out;
		}
		if(isset($data[$typeId]))
		{
			return $data[$typeId];
		}

		$table = JTable::getInstance('Type', 'CobaltTable');
		$table->load($typeId);

		$params = new JRegistry();
		$params->loadString($table->params);
		$table->params = $params;
		
	//	$table->description = JHtml::_('content.prepare', $table->description);

		if(!$table->id)
		{
			$this->setError('Type not exists');
			return FALSE;
		}
		$data[$typeId] = $table;

		return $data[$typeId];
	}

	public function validate($form, $data, $group = null){
		$user = JFactory::getUser();

		if($user->get('id') || isset($data['hidecaptcha']))
		{
			$data['captcha'] = 123;
		}
		else
		{
			if($data['captcha'] = JRequest::getVar('recaptcha_response_field'))
			{
				jimport('mint.recaptchalib');
				$conf = JComponentHelper::getParams('com_cobalt');

				$privatekey = $conf->get('recaptcha_private_key');
				$resp = recaptcha_check_answer ($privatekey,
				                                $_SERVER["REMOTE_ADDR"],
				                                JRequest::getVar("recaptcha_challenge_field"),
				                                JRequest::getVar("recaptcha_response_field"));

				if (!$resp->is_valid) {
					$this->setError(JText::sprintf('CREE_CAPTCHAWRONG', $resp->error ));
				}
			}
		}

		$fields = $data['fields'];
		settype($fields, 'array');

		$section = ItemsStore::getSection(JRequest::getInt('section_id'));
		$type = ItemsStore::getType($data['type_id']);
		$fields_list = JModel::getInstance('Fields', 'CobaltModel')->getFormFields($data['type_id']);

		$obj = $data;
		$fields_highlights = array();
		JArrayHelper::toObject($obj);

		foreach($fields_list as $field)
		{
			$value = @$fields[$field->id];
			$field->validate($value, $obj, $type, $section);
			if ($field->getErrors())
			{
				foreach($field->getErrors() as $err)
				{
					$this->setError($err);
				}
				$fields_highlights[] = $field->id;
			}
		}

		// highlight fields with errors
		if($fields_highlights)
		{
			JFactory::getApplication()->setUserState('com_cobalt.fieldhighlights', $fields_highlights);
		}

		$return = parent::validate($form, $data, $group);

		if($this->getErrors())
		{
			$return = false;
		}
		return $return;

	}
	/**
	 * Get the return URL.
	 *
	 * @return	string	The return URL.
	 * @since	1.6
	 */
	public function getReturnPage()
	{
		return base64_encode($this->getState('return_page'));
	}
}