<?php
/**
 * Cobalt by MintJoomla
 * a component for Joomla! 1.7 - 2.5 CMS (http://www.joomla.org)
 * Author Website: http://www.mintjoomla.com/
 * @copyright Copyright (C) 2012 MintJoomla (http://www.mintjoomla.com). All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */
defined('_JEXEC') or die();

jimport('joomla.application.component.modellist');

class CobaltModelOrders extends JModelList
{

	public function __construct($config = array())
	{
		if(empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'o.name',
				'section',
				'o.ctime',
				'o.status'
			);
		}

		parent::__construct($config);
	}

	public function getListQuery()
	{
		$user = JFactory::getUser();

		$db = $this->getDbo();
		$query = $db->getQuery(true);

		$query->select('o.*');
		$query->from('#__js_res_sales AS o');

		$query->select('s.name AS section');
		$query->leftJoin('#__js_res_sections AS s ON o.section_id = s.id');

		$query->select('r.title AS rtitle');
		$query->leftJoin('#__js_res_record AS r ON o.record_id = r.id');

		$query->select('u.username');
		$query->leftJoin('#__users AS u ON o.user_id = u.id');

		$search = $this->getState('filter.search', 0);
		if($search)
		{
			switch(substr($search, 0, 4))
			{
				case 'pid:':
					$where = 'o.record_id = '.(int)str_replace('pid:', '', $search);
					break;

				case 'bid:':
					$where = 'o.user_id = '.(int)str_replace('bid:', '', $search);
					break;

				case 'sid:':
					$where = 'o.saler_id = '.(int)str_replace('sid:', '', $search);
					break;

				default:
					$s1 = $db->Quote('%' . $db->getEscaped($search, true) . '%');
					$s = $db->Quote($db->getEscaped($search, true));

					$w[] = "o.gateway_id = {$s}";
					$w[] = 'o.name LIKE ' . $s1;
					$w[] = "o.user_id = " . (int)$s;
					$w[] = "u.username = {$s}";

					$where = "(" . implode(' OR ', $w) . ")";
			}

			$query->where($where);
		}

		$buyer = $this->getState('filter.buyer', 0);
		if($buyer)
		{
			$query->where('o.user_id = ' . $buyer);
		}

		$saler = $this->getState('filter.saler', 0);
		if($saler && ! $this->isSuperUser())
		{
			$query->where('o.saler_id = ' . $saler);
		}

		$status = $this->getState('filter.status');
		if($status)
		{
			$query->where('o.status = ' . (int)$status);
		}

		$section = $this->getState('filter.section');
		if($section)
		{
			$query->where('o.section_id = ' . (int)$section);
		}

		$orderCol = $this->state->get('list.ordering');
		$orderDirn = $this->state->get('list.direction');
		$query->order($db->getEscaped($orderCol . ' ' . $orderDirn));
		// 		echo $query;
		return $query;
	}

	public function getSections()
	{
		$db = $this->getDbo();

		$sql = "SELECT DISTINCT section_id FROM #__js_res_sales";

		if($this->getState('filter.saler', 0) && ! $this->isSuperUser())
		{
			$sql .= " WHERE saler_id = " . $this->getState('filter.saler', 0);
		}
		elseif($this->getState('filter.buyer', 0))
		{
			$sql .= " WHERE user_id = " . $this->getState('filter.buyer', 0);
		}

		$query = $db->getQuery(true);
		$query->select('s.name AS text, s.id AS value');
		$query->from('#__js_res_sections AS s');
		$query->where('s.published = 1');
		$query->where("s.id IN ({$sql})");

		$db->setQuery($query);

		$sections = $db->loadObjectList();
		$user_id = JFactory::getUser()->get('id');
		foreach($sections as $key => $value)
		{
			if(! MECAccess::isModerator($user_id, $value->value))
			{
				//unset($sections[$key]);
			}
		}
		return $sections;
	}

	public function isSuperUser()
	{
		$app = JFactory::getApplication();
		$menu = $app->getMenu();
		$active = $menu->getActive();

		if(!@$active->params instanceof JRegistry)
		{
			return false;
		}

		$allow_users = $active->params->get('allow_users', false);
		$user_ids = explode(',', $allow_users);

		JArrayHelper::toInteger($user_ids);
		ArrayHelper::clean_r($user_ids);

		return (in_array(JFactory::getUser()->get('id'), $user_ids));
	}

	public function getSection()
	{
		$section = $this->getState('filter.section');
		if($section)
		{
			return ItemsStore::getSection($this->getState('filter.section'));
		}

		return null;
	}

	public function getStoreId($id = NULL)
	{
		$id .= ':' . $this->getState('list.start');
		$id .= ':' . $this->getState('list.limit');
		$id .= ':' . $this->getState('list.ordering');
		$id .= ':' . $this->getState('list.direction');

		return md5($this->context . ':' . $id);
	}

	protected function populateState($ordering = null, $direction = null)
	{
		$app = JFactory::getApplication();

		$search = $app->getUserStateFromRequest($this->context . '.filter.search', 'filter_search');
		$this->setState('filter.search', $search);

		$status = $app->getUserStateFromRequest($this->context . '.filter.status', 'filter_status', '', 'string');
		$this->setState('filter.status', $status);

		$section = $app->getUserStateFromRequest($this->context . '.filter.section', 'filter_section', '', 'string');
		$this->setState('filter.section', $section);

		$buyer = $app->getUserStateFromRequest($this->context . '.filter.buyer', 'filter_buyer', '', 'int');
		$this->setState('filter.buyer', $buyer);

		$saler = $app->getUserStateFromRequest($this->context . '.filter.saler', 'filter_saler', '', 'int');
		$this->setState('filter.saler', $saler);

		parent::populateState('o.ctime', 'desc');
	}

}
