<?php
/**
 * Cobalt by MintJoomla
 * a component for Joomla! 1.7 - 2.5 CMS (http://www.joomla.org)
 * Author Website: http://www.mintjoomla.com/
 * @copyright Copyright (C) 2012 MintJoomla (http://www.mintjoomla.com). All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */
defined('_JEXEC') or die();

jimport('joomla.application.component.modelitem');
require_once JPATH_ROOT.DS.'components'.DS.'com_cobalt'.DS.'library'.DS.'php'.DS.'helper.php';

class CobaltModelRecord extends JModelItem
{
	protected $_context = 'com_cobalt.record';

	protected $_item = array();
	protected $_commetns = array();
	
	static $sortable = array();

	protected function populateState($ordering = null, $direction = null)
	{
		$app = JFactory::getApplication('site');

		// Load state from the request.
		$pk = JRequest::getInt('id');
		$this->setState('com_cobalt.record.id', $pk);

	}

	public function &getItem($pk = null)
	{
		// Initialise variables.
		$pk = (!empty($pk)) ? $pk : (int)$this->getState('com_cobalt.record.id');
		$user = JFactory::getUser();

		if (isset($this->_item[$pk]))
		{
			return $this->_item[$pk];
		}

		try
		{
			$db = $this->getDbo();
			$query = $db->getQuery(true);

			$query->select('r.*');
			$query->from('#__js_res_record AS r');

			/* $query->select('u.name AS name, u.username AS username');
			$query->join('LEFT', '#__users AS u on u.id = r.user_id'); */

			$query->select('c.name AS ucatname');
			$query->join('LEFT', '#__js_res_category_user AS c on c.id = r.ucatid');

			$query->where('r.id = ' . (int)$pk);

			if ($user->get('id'))
			{
				$query->select('(SELECT record_id FROM #__js_res_favorite WHERE record_id = r.id AND user_id = ' . $user->get('id') . ') as bookmarked');
				$query->select('(SELECT id FROM #__js_res_subscriptions WHERE record_id = r.id AND user_id = ' . $user->get('id') . ') as subscribed');
			}

			$db->setQuery($query);

			$data = $db->loadObject();

			if ($error = $db->getErrorMsg())
			{
				throw new Exception($error);
			}

			if (empty($data))
			{
				return JError::raiseError(404, JText::_('CERR_RECNOTFOUND'));
			}

			JRequest::setVar('section_id', $data->section_id);
			JRequest::setVar('type_id', $data->type_id);

			$this->_item[$pk] = $data;
		}
		catch ( JException $e )
		{
			if ($e->getCode() == 404)
			{
				// Need to go thru the error handler to allow Redirect to work.
				JError::raiseError(404, $e->getMessage());
			}
			else
			{
				$this->setError($e);
				$this->_item[$pk] = false;
			}
		}

		return $this->_item[$pk];
	}

	public function _prepareItem($data, $client = 'full')
	{

		static $fields = array(), $fields_model = NULL, $user = NULL;

		if (!$user)
		{
			$user = JFactory::getUser();
		}
		if (!$fields_model)
		{
			$fields_model = JModel::getInstance('Fields', 'CobaltModel');
		}
		$type = ItemsStore::getType($data->type_id);
		$section = ItemsStore::getSection($data->section_id);
		
		$data->ctime = JFactory::getDate($data->ctime);
		$data->mtime = JFactory::getDate($data->mtime);
		
		if($section->params->get('general.marknew'))
		{
			$data->new = (boolean)empty($data->new);
			if($data->user_id && ($data->user_id == $user->get('id')))
			{
				$data->new = false;
			}
			
			/*if($user->get('id'))
			{
				$date = JFactory::getDate($user->get('lastvisitDate'))->toUnix();
				if($date > $data->ctime->toUnix())
				{
					$data->new = false;
				}
			}
			*/
		}
		
		$data->params = new JRegistry($data->params);

		$data->type_name = $type->name;

		$data->categories = json_decode($data->categories, true);
		settype($data->categories, 'array');

		$data->categories_links = array();
		$data->category_id = 0;
		$category_links = $cat_ids = array();
		foreach($data->categories as $cat_id => $title)
		{
			$data->category_id = $cat_id;
			$cat_ids[] = $cat_id;
			$category_links[] = JHtml::link(JRoute::_(Url::records($section, $cat_id)), $title);
		}
		$data->categories_links = $category_links;

		JArrayHelper::toInteger($cat_ids);
		if (JRequest::getInt('cat_id') && in_array(JRequest::getInt('cat_id'), $cat_ids))
		{
			$category_id = JRequest::getInt('cat_id');
		}
		else
		{
			$category_id = array_shift($cat_ids);
		}

		$data->url = Url::record($data, $type, $section, $category_id);

		$robots = $type->params->get('submission.robots');

		$data->nofollow = substr_count($robots, 'noindex');
		
		if ($data->extime == '0000-00-00 00:00:00')
		{
			$data->extime = NULL;
		}
		else
		{
			$data->extime = JFactory::getDate($data->extime);
			if ($data->extime->toUnix() < strtotime("+5 day") && $data->exalert == 0)
			{
				$this->_db->setQuery("UPDATE #__js_res_record SET exalert = 1 WHERE id = " . $data->id);
				$this->_db->query();
				CEventsHelper::notify('record', CEventsHelper::_RECORD_EXPIRED, $data->id, $data->section_id, 0, 0, 0, $data);
			}
		}
		$data->ucatname_link = '';
		if ($data->ucatid)
		{
			$data->ucatname_link = JHtml::link(JRoute::_(URL::usercategory_records($data->user_id, $section, $data->ucatid)), $data->ucatname);
		}

		$data->tags = json_decode($data->tags, true);
		ArrayHelper::clean_r($data->tags);

		$fields[$data->id] = $fields_model->getRecordFields($data, 'all');
		$sorted = $final = $keyed = array();
		foreach($fields[$data->id] as $key => $field)
		{
			if($field->params->get('params.sortable'))
			{
				self::$sortable[$field->key] = $field;
			}
			if($client == 'feed' && !$field->params->get('core.show_feed', 0)) continue;
			if($client == 'list' && !$field->params->get('core.show_intro', 0)) continue;
			if($client == 'full' && !$field->params->get('core.show_full', 0)) continue;

			if (!in_array($field->params->get('core.field_view_access'), $user->getAuthorisedViewLevels()))
			{
				if (!trim($field->params->get('core.field_view_message')))
				{
					continue;
				}
				else
				{
					$result = $field->params->get('core.field_view_message');
				}
			}
			else
			{
				if (!CEmeraldHelper::allowField('display', $field, $data->user_id, $section))
				{
					if ($field->params->get('emerald.field_display_subscription'))
					{
						$result = JText::_($field->params->get('emerald.field_display_subscription_msg'));
					}
					else continue;
				}
				else
				{
					CEmeraldHelper::countLimit('field', 'display', $field, $data->user_id);

					if (!CEmeraldHelper::allowField('view', $field, $data->user_id, $section))
					{
						if ($field->params->get('emerald.field_view_subscription'))
						{
							$plans = CEmeraldHelper::getSubscrList($field->params->get('emerald.field_view_subscription'), $field->params->get('emerald.subscr_idd', JRequest::getInt('Itemid')));
							$result = trim(JText::_($field->params->get('emerald.field_view_subscription_msg')));
							if($result)
							{
								$result .= ' '.count($plans) > 1 ? '<ul class="plans-list"><li>'.implode("</li><li>", $plans).'</li></ul>' : implode("", $plans);
							}
						}
						else continue;
					}
					else
					{

						CEmeraldHelper::countLimit('field', 'view', $field, $user->id);

						$method = $client == 'full' ? 'onRenderFull' : 'onRenderList';
						$result = $field->$method($data, $type, $section);
					}
				}
			}

			$result = trim($result);
			if (!$result)
				continue;
			

			$field->result = $result;
			$keyed[$field->key] = $field;
			$final[$field->id] = $field;
			$sorted[$field->group_title][$field->key] = $field;
		}
		
		$data->fields_by_id = $final;
		$data->fields_by_groups = $sorted;
		$data->fields_by_key = $keyed;
		
		if ($data->featured == 1)
		{
			$data->ftime = JFactory::getDate($data->ftime);
			if ($data->ftime->toUnix() < time())
			{
				$sql = "UPDATE #__js_res_record SET featured = 0, ftime = '0000-00-00' WHERE id = " . $data->id;
				$this->_db->setQuery($sql);
				$this->_db->query();
				CEventsHelper::notify('record', CEventsHelper::_RECORD_FEATURED_EXPIRED, $data->id, $data->section_id, 0, 0, 0, $data);
			}
		}

		$data->rating = RatingHelp::loadMultiratings($data, $type, $section);
		$data->controls = $this->_controls($data, $type, $section);
		$data->controls_notitle = $this->_controls($data, $type, $section, true);

		return $data;
	}

	private function _controls($record, $type, $section, $notitle = false)
	{
		$user = JFactory::getUser();
		$view = JRequest::getString('view');
		if($notitle)
		{
			$patern = '<a href="%s"><img border="0" src="' . JURI::root() . 'media/mint/icons/16/%s" alt="%s" align="absmiddle" title="%s" /></a>';
			$confirm_patern = '<a href="%s" onclick="javascript:if(!confirm(\'%s\')){return false;}"><img border="0" src="' . JURI::root() . 'media/mint/icons/16/%s" alt="%s" align="absmiddle" title="%s" /></a>';
		}
		else
		{
			$patern = '<a href="%s"><img border="0" src="' . JURI::root() . 'media/mint/icons/16/%s" alt="%s" align="absmiddle" /> %s</a>';
			$confirm_patern = '<a href="%s" onclick="javascript:if(!confirm(\'%s\')){return false;}"><img border="0" src="' . JURI::root() . 'media/mint/icons/16/%s" alt="%s" align="absmiddle" /> %s</a>';
		}
		$out = array();
		if (!$user->get('id'))
		{
			return array();
		}

		if (MECAccess::allowDelete($record, $type, $section) && $view == 'records')
		{
			$out[] = sprintf($confirm_patern, Url::task('records.delete', $record->id), addslashes(JText::_('CCONFIRMDELET_1')),'minus-circle.png', JText::_('CDELETE'), JText::_('CDELETE'));
		}
		if (MECAccess::allowDelete($record, $type, $section) && $view == 'record')
		{
			$out[] = sprintf($confirm_patern, Url::task('records.delete', $record->id, Url::records($record->section_id, $record->category_id)), addslashes(JText::_('CCONFIRMDELET_1')),'minus-circle.png', JText::_('CDELETE'), JText::_('CDELETE'));
		}
		if (MECAccess::allowEdit($record, $type, $section))
		{
			$out[] = sprintf($patern, Url::edit($record), 'pencil.png', JText::_('CEDIT'), JText::_('CEDIT'));
		}
		if (MECAccess::allowArchive($record, $type, $section))
		{
			$out[] = sprintf($patern, Url::task('records.sarchive', $record->id), 'wooden-box.png', JText::_('CARCHIVE'), JText::_('CARCHIVE'));
		}

		if (MECAccess::allowExtend($record, $type, $section))
		{
			$out[] = sprintf($patern, Url::task('records.prolong', $record->id), 'clock--plus.png', JText::sprintf('CPROLONG', $type->params->get('properties.default_extend')), JText::sprintf('CPROLONG', $type->params->get('properties.default_extend')));
		}

		if (MECAccess::allowFeatured($record, $type, $section))
		{
			$out[] = sprintf($patern, Url::task('records.sfeatured', $record->id), 'crown.png', JText::sprintf('CMAKEFEATURE', $type->params->get('emerald.type_feature_subscription_time', 10)),
				 JText::sprintf('CMAKEFEATURE', $type->params->get('emerald.type_feature_subscription_time', 10)));
		}

		if (MECAccess::allowCommentBlock($record, $type, $section))
		{
			$enabled = $record->params->get('comments.comments_access_post', $type->params->get('comments.comments_access_post'));
			$out[] = sprintf($patern, Url::task('records.' . ($enabled ? 'commentsdisable' : 'commentsenable'), $record->id), ($enabled ? 'balloon--minus.png' : 'balloon--plus.png'), ($enabled ? JText::_('CDISABCOMM') : JText::_('CENABCOMM')), ($enabled ? JText::_('CDISABCOMM') : JText::_('CENABCOMM')));
		}

		if (MECAccess::allowPublish($record, $type, $section))
		{
			$out[] = sprintf($patern, Url::task('records.' . ($record->published ? 'sunpub' : 'spub'), $record->id), ($record->published ? 'cross-circle.png' : 'tick.png'), ($record->published ? JText::_('CUNPUB') : JText::_('CPUB')), ($record->published ? JText::_('CUNPUB') : JText::_('CPUB')));
		}

		if (MECAccess::allowHide($record, $type, $section))
		{
			$out[] = sprintf($patern, Url::task('records.' . ($record->hidden ? 'sunhide' : 'shide'), $record->id), ($record->hidden ? 'eye-half0.png' : 'eye-half.png'), ($record->hidden ? JText::_('CUNHIDE') : JText::_('CHIDE')), ($record->hidden ? JText::_('CUNHIDE') : JText::_('CHIDE')));
		}

		if (MECAccess::allowModerate($record, $type, $section))
		{
			$url = 'index.php?option=com_cobalt&view=moderator&user_id=' . $record->user_id . '&section_id=' . $section->id . '&return=' . base64_encode(JFactory::getURI()->toString());
			$out[] = sprintf($patern, JRoute::_($url), 'user-share.png', JText::_('CSETMODER'), JText::_('CSETMODER'));
		}

		if ($out)
		{
			return $out;
		}
	}

	public function hit($pk = 0, $section_id = NULL)
	{
		$user = JFactory::getUser();
		$config = JFactory::getConfig();

		$cookie_domain = $config->get('cookie_domain', '');
		$cookie_path = $config->get('cookie_path', '/');

		$hits = JTable::getInstance('Hits', 'CobaltTable');

		if ($user->get('id'))
		{
			$data = array('user_id' => $user->get('id'), 'record_id' => $pk);
		}
		else
		{
			$data = array('ip' => $_SERVER['REMOTE_ADDR'], 'record_id' => $pk);
		}
		$hits->load($data);

		if ($hits->id)
		{
			return;
		}

		$data['section_id'] = JRequest::getInt('section_id', $section_id);
		$hits->bind($data);
		$hits->check();
		$hits->store();

		$db = $this->getDbo();
		$db->setQuery("UPDATE #__js_res_record SET hits = hits + 1 WHERE id = " . $pk);
		$db->query();

		return true;
	}

	public function onComment($id)
	{
		$this->_db->setQuery('UPDATE #__js_res_record SET comments = (SELECT COUNT(id) FROM #__js_res_comments WHERE record_id = ' . $id . ' AND published = 1) WHERE id = ' . $id);
		$this->_db->query();
		
		$record = $this->getItem($id);
		$record = ItemsStore::getRecord($id);
		$section = ItemsStore::getSection($record->section_id);
		static $fields_model = null; 
		if (!$fields_model)
		{
			$fields_model = JModel::getInstance('Fields', 'CobaltModel');
		}
		$fields = $fields_model->getRecordFields($record);
		foreach ($fields as $field)
		{			
			if(method_exists($field, 'onComment'))
				$field->onComment($record, $section);			
		}
		
	}
}
