<?php
/**
 * Cobalt by MintJoomla
 * a component for Joomla! 1.7 - 2.5 CMS (http://www.joomla.org)
 * Author Website: http://www.mintjoomla.com/
 * @copyright Copyright (C) 2012 MintJoomla (http://www.mintjoomla.com). All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */

defined('_JEXEC') or die();

jimport('joomla.application.component.view');

/**
 * HTML Article View class for the Content component
 *
 * @package		Joomla.Site
 * @subpackage	com_content
 * @since		1.5
 */
class CobaltViewForm extends JView
{
	protected $form;
	protected $item;
	protected $return_page;
	protected $state;

	public function display($tpl = null)
	{
		// Initialise variables.
		$app = JFactory::getApplication();
		$doc = JFactory::getDocument();
		$user = JFactory::getUser();

		$this->state = $this->get('State');
		$this->item = $this->get('Item');

		if (!JRequest::getInt('section_id', @$this->item->section_id))
		{
			JError::raiseWarning(403, JText::_('CNOSECTION'));
			return FALSE;
		}

		if (!JRequest::getInt('type_id'))
		{
			$app->redirect('index.php?option=com_cobalt&view=types&section_id=' . JRequest::getInt('section_id'));
			return FALSE;
		}

		$this->type = $this->get('RecordType');
		$this->form = $this->get('Form');

		$this->params = $this->type->params;

		if($this->params->get('properties.item_title_limit', 0))
		{
			$this->form->setFieldAttribute('title', 'maxlength', $this->params->get('properties.item_title_limit', 0));
		}


		if(empty($this->item->id) && !in_array($this->params->get('submission.submission'), $user->getAuthorisedViewLevels()))
		{
			JError::raiseWarning(403, JText::_('CNOPERMISION'));
			$modal = '';
			if(JRequest::getInt('modal', false))
			{
				$modal = '&tmpl=component&modal=1';
			}
			$app->redirect(JRoute::_('index.php?option=com_users&return='.base64_encode(JFactory::getURI()->toString()).$modal));
			return FALSE;
		}

		$model_section = JModel::getInstance('Section', 'CobaltModel');
		$section = $model_section->getItem(JRequest::getInt('section_id'));

		if (empty($section->id))
		{
			JError::raiseWarning(403, JText::_('CNOSECTION'));
			return FALSE;
		}

		if (!in_array($this->type->id, $section->params->get('general.type')))
		{
			JError::raiseWarning(403, JText::_('CERRTYPENOTALLOWED'));
			return FALSE;
		}

		if (empty($this->item->id) && $section->params->get('general.record_submit_limit') > 0 && ($model_section->countUserRecords($section->id) >= $section->params->get('general.record_submit_limit')))
		{
			JError::raiseWarning(403, JText::sprintf('CMAXSUBMITREACHED', $section->params->get('general.record_submit_limit')));
			return FALSE;
		}

		if ($this->item->id && !MECAccess::allowEdit($this->item, $this->type, $section))
		{
			JError::raiseWarning(403, JText::_('CNOPERMEDIT'));
			$modal = '';
			if(JRequest::getInt('modal', false))
			{
				$modal = '&tmpl=component&modal=1';
			}
			$app->redirect(JRoute::_('index.php?option=com_users&return='.base64_encode(JFactory::getURI()->toString()).$modal));
			return FALSE;
		}

		CEmeraldHelper::allowType(($this->item->id ? 'edit' : 'submit'), $this->type, $this->item, $section, true);

		$model_fields = JModel::getInstance('Fields', 'CobaltModel');
		$fields = $model_fields->getFormFields(JRequest::getInt('type_id', @$this->item->type_id), @$this->item->id);
		$this->_prepareFields($fields, $section, $this->item);

		// Check for errors.
		if (count($errors = $this->get('Errors')))
		{
			JError::raiseWarning(403, implode("\n", $errors));
			return false;
		}

		if (empty($this->item->id))
		{
			$authorised = $user->authorise('tmpl_core.create', 'com_cobalt') || (count($user->getAuthorisedCategories('com_cobalt', 'tmpl_core.create')));
		}
		else
		{
			$authorised = $this->item->params->get('access-edit');
		}

		if (!empty($this->item))
		{
			$this->form->bind($this->item);
		}
		JRequest::setVar('cat_user_id', $this->item->user_id);

		$tmpl_params = new JRegistry();
		$file = dirname(__FILE__) . DS . 'tmpl' . DS . 'default_form_' . $this->params->get('properties.tmpl_articleform', 'default') . '.json';
		if(JFile::exists($file))
		{
			$file = JFile::read($file);
		}
		else
		{
			$file = array();
			JError::raiseNotice(404, JText::_('CPLSEDITTEMPLATE_AF'));
		}
		$tmpl_params->loadString($file);
		$this->tmpl_params = $tmpl_params;

		$file = dirname(__FILE__) . DS . 'tmpl' . DS . 'default_form_' . $this->params->get('properties.tmpl_articleform', 'default') . '.css';
		if (JFile::exists($file))
		{
			$doc->addStyleSheet(JURI::root() . 'components/com_cobalt/views/form/tmpl/default_form_' . $this->params->get('properties.tmpl_articleform', 'default') . '.css');
		}

		$core_fields_all = array("ctime", "extime", "ftime", "hidden", "langs", "votes", "hits", "ip", "votes_result", "featured", "user_id", "archive", "access", "published");

		$core_fields = $core_admin_fields = array();

		foreach($core_fields_all as $key => $field)
		{
			$this->form->setFieldAttribute($field, 'label', $tmpl_params->get('tmpl_core.form_label_' . $field));
			if ($tmpl_params->get('tmpl_core.form_show_' . $field) == 1 || $tmpl_params->get('submission.submission') == $tmpl_params->get('tmpl_core.form_show_' . $field))
			{
				$core_fields[] = $field;
			}
			else
			{
				if (in_array($tmpl_params->get('tmpl_core.form_show_' . $field), $user->getAuthorisedViewLevels()))
				{
					$core_admin_fields[] = $field;
				}
			}
		}

		$this->core_fields = $core_fields;
		$this->core_admin_fields = $core_admin_fields;

		$this->multirating = false;
		if ($this->type->params->get('properties.rate_access', 0) == -1)
		{
			$this->multirating = RatingHelp::loadFormMultiratings($this->item, $this->type, $section);
		}

		$this->meta = array();

		if (in_array($tmpl_params->get('tmpl_core.form_show_meta_descr'), $user->getAuthorisedViewLevels()))
		{
			$this->meta['Meta Description'] = 'meta_descr';
		}
		if (in_array($tmpl_params->get('tmpl_core.form_show_meta_key'), $user->getAuthorisedViewLevels()))
		{
			$this->meta['Meta Keywords'] = 'meta_key';
		}
		if (in_array($tmpl_params->get('tmpl_core.form_show_meta_robots'), $user->getAuthorisedViewLevels()))
		{
			$this->meta['Robots'] = 'meta_index';
		}
		if (in_array($tmpl_params->get('tmpl_core.form_show_alias'), $user->getAuthorisedViewLevels()))
		{
			$this->meta['Url Alias'] = 'alias';
		}

		$this->user = $user;

		if (in_array($this->params->get('submission.allow_category'), $user->getAuthorisedViewLevels()))
		{
			if (is_array($section->params))
			{
				$params = new JRegistry();
				$params->loadArray($section->params);
				$section->params = $params;
			}
			$this->section = $section;
			$catsel_params = new JRegistry();
			$file = dirname(__FILE__) . DS . 'tmpl' . DS . 'default_category_' . $this->params->get('properties.tmpl_categoryselect', 'default') . '.json';
			if(JFile::exists($file))
			{
				$file = JFile::read($file);
			}
			else
			{
				$file = array();
				JError::raiseNotice(404, JText::_('CPLSEDITTEMPLATE_CS'));
			}
			$catsel_params->loadString($file);
			$this->catsel_params = $catsel_params;
			if (JFile::exists(dirname(__FILE__) . DS . 'tmpl' . DS . 'default_category_' . $this->params->get('properties.tmpl_categoryselect', 'default') . '.css'))
			{
				$doc->addStyleSheet(JURI::root() . 'components/com_cobalt/views/form/tmpl/default_category_' . $this->params->get('properties.tmpl_categoryselect', 'default') . '.css');
			}

			$data = JFactory::getApplication()->getUserState('com_cobalt.edit.form.data', array());

			if (!isset($data['category']) && !empty($this->item->categories))
			{
				$data['category'] = array_keys($this->item->categories);
			}
			elseif(!isset($data['category']) && !$this->item->id && JRequest::getInt('cat_id'))
			{
				$data['category'] = JRequest::getInt('cat_id');
			}

			$this->default_categories = @$data['category'];
		}

		$this->section = $section;
		$this->category = null;
		if($app->input->getInt('cat_id'))
		{
			$this->category = ItemsStore::getCategory($app->input->getInt('cat_id'));
		}

		$this->_prepareDocument();

		parent::display($tpl);
	}

	private function _prepareDocument()
	{
		$app	= JFactory::getApplication();
		$doc = JFactory::getDocument();
		$menus	= $app->getMenu();
		$pathway = $app->getPathway();
		$title = false;
		if($this->item->id)
		{
			$title = JText::sprintf('CTEDIT', $this->escape($this->type->name), $this->item->title);
			$pathway->addItem($title);
		}
		else
		{
			$title = JText::sprintf('CTSUBMIT', $this->escape($this->type->name));
			$pathway->addItem($title);
		}

		$this->appParams = $app->getParams();
		// Because the application sets a default page title,
		// we need to get it from the menu item itself
		$menu = $menus->getActive();
		if ($menu)
		{
			$title .= ' - '.$menu->params->get('page_title', $menu->title);
			$this->appParams->def('page_heading', $title);
		}
		else
		{
			$title .= ' - '.$this->section->name;
		}
		// Check for empty title and add site name if param is set
		if (empty($title)) {
			$title = $app->getCfg('sitename');
		}
		elseif ($app->getCfg('sitename_pagetitles', 0) == 1) {
			$title = JText::sprintf('JPAGETITLE', $app->getCfg('sitename'), $title);
		}
		elseif ($app->getCfg('sitename_pagetitles', 0) == 2) {
			$title = JText::sprintf('JPAGETITLE', $title, $app->getCfg('sitename'));
		}
		if (empty($title)) {
			$title = $this->item->title;
		}
		$doc->setTitle($title);
	}

	private function _prepareFields($fields, $section, $item)
	{
		$sorted = $fg = array();
		$user = JFactory::getUser();

		foreach($fields as $key => $field)
		{
			$result = $msg = NULL;
			if (!$result = $field->getInput($this->item))
			{
				continue;
			}

			if (!$this->item->id)
			{
				if (!in_array($field->params->get('core.field_submit_access'), $user->getAuthorisedViewLevels()))
				{
					if (!trim($field->params->get('core.field_submit_message')))
					{
						continue;
					}
					else
					{
						$msg = $field->params->get('core.field_submit_message');
					}
				}
			}
			else
			{
				if (!in_array($field->params->get('core.field_edit_access'), $user->getAuthorisedViewLevels()))
				{
					if (!trim($field->params->get('core.field_edit_message')))
					{
						continue;
					}
					else
					{
						$msg = $field->params->get('core.field_edit_message');
					}
				}
			}

			$method = $this->item->id ? 'edit' : 'submit';
			if (!CEmeraldHelper::allowField($method, $field, $item->user_id, $section))
			{
				if ($field->params->get('emerald.field_'.$method.'_subscription'))
				{
					$plans = CEmeraldHelper::getSubscrList($field->params->get('emerald.field_'.$method.'_subscription'), $field->params->get('emerald.subscr_iid', JRequest::getInt('Itemid')));
					$msg = JText::_($field->params->get('emerald.field_'.$method.'_subscription_msg')) . implode($plans);
				}
				else continue;
			}

			if (!$msg)
			{
				$field->result = $result;
				$field->js = $field->onJSValidate();
			}
			else
			{
				$field->result = $msg;
				$field->js = '';
			}
			$sorted[(int)$field->group_id][$key] = $field;
			$fg[$field->group_id]['name'] = $field->group_title;
			$fg[$field->group_id]['descr'] = $field->group_descr;

		}

		$this->fields = $fields;
		$this->sorted_fields = $sorted;
		$this->field_groups = $fg;
	}
}
