<?php
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die();

jimport('joomla.application.component.view');

class CobaltViewRecords extends JView
{
	function display()
	{
		$app = JFactory::getApplication();
		$doc = JFactory::getDocument();
		$user = JFactory::getUser();
		$model = $this->getModel();

		$section_id = JRequest::getInt('section_id', 0);
		$cat_id = JRequest::getInt('cat_id', 0);

		$this->models['category'] 	= JModel::getInstance('Category', 'CobaltModel');
		$this->models['categories'] = JModel::getInstance('Categories', 'CobaltModel');
		$this->models['section'] 	= JModel::getInstance('Section', 'CobaltModel');
		$this->models['record'] 	= JModel::getInstance('Record', 'CobaltModel');
		$fields_model = JModel::getInstance('Fields', 'CobaltModel');

		if(!JRequest::getInt('section_id'))
		{
			JError::raiseWarning(404, JText::_('CNOSECTION'));
			return;
		}

		// ----- GET SECTION ------
		$section = $this->models['section']->getItem(JRequest::getInt('section_id'));

		if ($section->published == 0)
		{
			JError::raise(E_WARNING, 403, JText::_('CERR_SECTIONUNPUB'));
			return;
		}

		if(!in_array($section->access, $user->getAuthorisedViewLevels()))
		{
			JError::raise(E_WARNING, 403, JText::_($section->params->get('general.access_msg')));
			return;
		}
		$this->section = $section;
		$this->section->params->get('general.featured_first', 0);
		
		$model->section = $this->section;

		// --- GET CATEGORY ----
		$category = $this->models['category']->getEmpty();
		if(JRequest::getInt('cat_id'))
		{
			$category = $this->models['category']->getItem(JRequest::getInt('cat_id'));
			if(!isset($category->id))
			{
				JError::raiseNotice(404, JText::_('CCATNOTFOUND'));
				$category = $this->models['category']->getEmpty();
			}
			if($category->id && ($category->section_id != $section->id))
			{
				JError::raiseNotice(403, JText::_('CCATWRONGSECTION'));
				$category = $this->models['category']->getEmpty();
			}
			JRequest::setVar('cat_id', $category->id);
		}
		$this->category = $category;

		$items = NULL;
		$items = $this->get('Items');

		if(JRequest::getVar('type') == 'xml')
		{
			$files = array();
			foreach ($items AS $key => $row)
			{
				$files[$key] = "\t<item>\n";
				$files[$key] .= "\t\t<id>{$row->id}</id>\n";
				$files[$key] .= "\t\t<create_date>{$row->ctime}</create_date>\n";
				$files[$key] .= "\t\t<modify_date>{$row->mtime}</modify_date>\n";
				$files[$key] .= "\t\t<title>".htmlentities($row->title)."</title>\n";
				$fields = $fields_model->getRecordFields($row, 'feed');
				$type = ItemsStore::getType($row->type_id);
				$files[$key] .= "\t\t<fields>\n";
				foreach ($fields AS $k => $field)
				{
					$tag = JString::strtolower($field->title);
					$tag = preg_replace("/[^a-z_-]/iU", "_", $tag);
					while (strstr($tag, "__"))
						$tag = str_replace("__", "_", $tag);
					$tag = $field->params->get('core.field_xml_tag_name', $tag);
					$result = $field->onRenderList($row, $type, $section);
					$files[$key] .= "\t\t<{$tag}><![CDATA[	".$result."  ]]></$tag>\n";
				}
				$files[$key] .= "\t\t</fields>\n";
				$files[$key] .= "\t</item>\n";
			}
			JResponse::setHeader('content-type', 'text/xml');
			JResponse::sendHeaders();
			echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>
			<items>\n".implode("\n", $files)."</items>";
			exit;
		}

		JRequest::setVar('limit', $app->getCfg('feed_limit'));
		$feedEmail	= (@$app->getCfg('feed_email')) ? $app->getCfg('feed_email') : 'author';

		$link = JRoute::_(Url::records($section, $category->id));
		$doc->link = $this->escape(JRoute::_($link));
		$doc->title = $section->title;
		$doc->description = JText::sprintf('CRSSFEEDRECORDS', $section->name);

		foreach ($items as $row)
		{
			// strip html from feed item title
			$title = $this->escape($row->title);
			$title = html_entity_decode($title, ENT_COMPAT, 'UTF-8');

			$type = ItemsStore::getType($row->type_id);
			$url = $this->escape(JRoute::_(Url::record($row, $type, $section)));

			$description = array();
			$fields = $fields_model->getRecordFields($row, 'feed');
			foreach ($fields AS $k => $field)
			{
				$result = $field->onRenderList($row, $type, $section);
				if($field->params->get('core.show_lable') < 2)
				{
					$description[] = sprintf('<tr valign="top"><td colspan="2">%s</td></tr>', $result);
				}
				else
				{
					$description[] = sprintf('<tr valign="top"><td><b>%s</b>:</td><td>%s</td></tr>', $field->title, $result);
				}
			}

			$description = '<table>'.implode(" ", $description).'</table>';
			$author			= CCommunityHelper::getName($row->user_id, $row->section_id, true);
			$date			= JFactory::getDate($row->ctime)->toISO8601();

			// load individual item creator class
			$item = new JFeedItem();
			$item->title		= $title;
			$item->link			= $url;
			$item->description	= $description;
			$item->date			= $date;

			$item->author		= $author;
			if ($feedEmail == 'site') {
				$item->authorEmail = $app->getCfg('mailfrom');
			}
			else {
				$item->authorEmail = JFactory::getUser($row->user_id)->get('email');
			}

			// loads item info into rss array
			$doc->addItem($item);
		}
	}
}
?>