<?php
/**
 * Cobalt by MintJoomla
 * a component for Joomla! 1.7 - 2.5 CMS (http://www.joomla.org)
 * Author Website: http://www.mintjoomla.com/
 * @copyright Copyright (C) 2012 MintJoomla (http://www.mintjoomla.com). All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */

defined('_JEXEC') or die();
require_once JPATH_ROOT.DS.'components/com_cobalt/library/php/fields/cobaltfield.php';

class JFormFieldCTextarea extends CFormField
{
	
	public function getInput()
	{
		$params = $this->params;
		$doc = JFactory::getDocument();
		
		$max_length = (int)$params->get('params.maxlen', 0);
		$text = ($this->value ? $this->value : $params->get('params.default_value'));
		
		if($max_length > 0)
		{
			$text = JString::substr($text, 0, $max_length);
		}
		$text = htmlspecialchars(stripslashes($text), ENT_QUOTES, 'UTF-8');
		
		$this->value = $text;
		
		return $this->_display_input();
		
	}
	
	public function onJSValidate()
	{
	}
	
	public function validate($value, $record, $type, $section)
	{
		$this->_filter($value);
		
		return parent::validate($value, $record, $type, $section);
	}
	
	public function onPrepareFullTextSearch($value, $record, $type, $section)
	{
		return strip_tags($this->onPrepareSave($value, $record, $type, $section));
	}
	
	public function onPrepareSave($value, $record, $type, $section)
	{
		return $value;
	}
	
	public function onRenderFull($record, $type, $section)
	{
		$this->value = $this->_filter($this->value);
		return $this->_display_output('full', $record, $type, $section);
	}
	
	public function onRenderList($record, $type, $section)
	{
		$value = $out = $this->_filter($this->value);
		if($this->params->get('params.intro', 0) > 0)
		{
			$out = HTMLFormatHelper::substrHTML($value, $this->params->get('params.intro'));
		}
		
		if(strlen($value) > strlen($out))
		{
			$out .= $this->params->get('params.seemore');
		}
		
		$this->value = $out;
		
		return $this->_display_output('list', $record, $type, $section);
	}
	private function _filter($value)
	{
		$value = trim($value);
		if($this->params->get('params.allow_html', 0) == 0)
		{
			// preserve markdown <http://www.kg> syntax
			$pattern = "/<(http[^>]*)>/iU";
			$value = preg_replace($pattern, "[\\1](\\1)", $value);
			
			// convert raw HTML to text;
			$value = htmlspecialchars($value, ENT_COMPAT, 'UTF-8');
		}
		
		if($this->params->get('params.markdown'))
		{
			jimport('mint.markdown');
			$value = Markdown($value);
		}
		else
		{
			$value = nl2br($value);
		}
		
		if($this->params->get('params.bbcode'))
		{
			$value = HTMLFormatHelper::bb2html($value);
		}
		
		
		$len = JString::strlen(str_replace(array( '<?php', '?>', '&amp;', '<hr>'), array('&lt;?php', '?&gt;', '&', '<hr />'), $value));
				
		if ($this->params->get('params.allow_html', 1) == 2)
		{
			$tags = explode(',', $this->params->get('params.filter_tags'));
			$attr = explode(',', $this->params->get('params.filter_attr'));
			ArrayHelper::trim_r($tags);
			ArrayHelper::trim_r($attr);
			ArrayHelper::clean_r($tags);
			ArrayHelper::clean_r($attr);
			$tag_mode = $this->params->get('params.tags_mode', 0);
			$attmode = $this->params->get('params.attr_mode', 0);

			$value = str_replace(array('&lt;', '&gt;', '<?php', '?>'), array('^^^', '@@@', '^@^', '@^@'), $value);
			$value = JFilterInput::getInstance($tags, $attr, $tag_mode, $attmode)->clean($value);
			$value = str_replace(array( '^@^', '@^@', '^^^', '@@@'), array( '&lt;?php', '?&gt;', '&lt;', '&gt;'), $value);
			
			$len1 = JString::strlen($value);
			if ($len != $len1)
			{
				$this->setError(JText::sprintf('TA_ENTEREDTAGSATTRSNOTALLOWED', $this->label));
			}
		}
		
		if($this->params->get('params.prepare', 1))
		{
			$value = JHtml::_('content.prepare', $value);
		}
		
		return $value;
	}
}
